/***************************************************************
 * Name:      wxiscanabout.cpp
 * Purpose:   Defines About Box Class.
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-06-12
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

#ifndef WX_PRECOMP
	//(*InternalHeadersPCH(wxIScanAbout)
	#include <wx/string.h>
	#include <wx/intl.h>
	//*)
#endif
//(*InternalHeaders(wxIScanAbout)
#include <wx/button.h>
//*)

// Private headers
#include "wxiscan.h"
#include "wxiscanabout.h"


//(*IdInit(wxIScanAbout)
const long wxIScanAbout::ID_HTMLWINDOW = wxNewId();
//*)


//////////////////////////////////////////////////////////
// Strings and other (global) constants
//
const wxString cstrAppName=            wxT( PRODUCT_NAME );
const wxString cstrAppDescription=     wxT( FILE_DESCRIPTION );
const wxString cstrVendor=             wxT( COMPANY_NAME );
const wxString cstrStatusCopyRight=    wxT( LEGAL_COPYRIGHT );
const wxString cstrVersionString=      wxString::Format( wxT( "%d.%d.%d (Build %d)" ),
                                                                AutoVersion::MAJOR,
                                                                AutoVersion::MINOR,
                                                                AutoVersion::BUILD,
                                                                AutoVersion::BUILDS_COUNT );
const wxString cstrWebSite=            wxT( WEB_SITE );
const wxString cstrWxVersionString=    wxVERSION_STRING;


//////////////////////////////////////////////////////////
// Class wxAboutHtmlWindow
//
class wxAboutHtmlWindow : public wxHtmlWindow
{
  public:
    wxAboutHtmlWindow(
                    wxWindow *poParent,
                    wxWindowID nId = wxID_ANY,
                    const wxPoint& oPos= wxDefaultPosition,
                    const wxSize& oSize= wxDefaultSize,
                    long nStyle= wxHW_DEFAULT_STYLE,
                    const wxString& strName= wxT( "wxAboutHtmlWindow" )
                )
      : wxHtmlWindow( poParent, nId, oPos, oSize, nStyle, strName )
    {
    }

  private:
    void OnLinkClicked( wxHtmlLinkEvent &oEvent )
    {
        ::wxLaunchDefaultBrowser( oEvent.GetLinkInfo().GetHref() );
    }

  private:
    DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE( wxAboutHtmlWindow, wxHtmlWindow )
    EVT_HTML_LINK_CLICKED( wxID_ANY, wxAboutHtmlWindow::OnLinkClicked )
END_EVENT_TABLE()


//////////////////////////////////////////////////////////
// Class wxIScanAbout
//
BEGIN_EVENT_TABLE( wxIScanAbout, wxDialog )
	//(*EventTable(wxIScanAbout)
	//*)
END_EVENT_TABLE()

wxIScanAbout::wxIScanAbout( wxWindow* parent, wxWindowID id )
{
	//(*Initialize(wxIScanAbout)
	wxFlexGridSizer* FlexGridSizer1;
	wxStdDialogButtonSizer* poStdDialogButtonSizer;
	wxAboutHtmlWindow* poHtmlWindow;
	
	Create(parent, id, _("About"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("id"));
	FlexGridSizer1 = new wxFlexGridSizer(2, 1, 0, 0);
	poHtmlWindow = new wxAboutHtmlWindow(this, ID_HTMLWINDOW, wxDefaultPosition, wxSize(450,420), wxHW_SCROLLBAR_AUTO, _T("ID_HTMLWINDOW"));
	FlexGridSizer1->Add(poHtmlWindow, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	poStdDialogButtonSizer = new wxStdDialogButtonSizer();
	poStdDialogButtonSizer->AddButton(new wxButton(this, wxID_OK, wxEmptyString));
	poStdDialogButtonSizer->Realize();
	FlexGridSizer1->Add(poStdDialogButtonSizer, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	SetSizer(FlexGridSizer1);
	FlexGridSizer1->Fit(this);
	FlexGridSizer1->SetSizeHints(this);
	//*)

	wxString strHtml= wxString::Format( wxT( "<html>"
                                               "<head>"
                                                 "<title>%s - About</title>"
                                               "</head>"
                                               "<body bgcolor=#FAF0E6>"
                                                 "<div>"
                                                   "<h3 align=\"center\">"
                                                        "%s"
                                                        "<img src=\"http://%s/pictures/nellresearch.png\">"
                                                        "%s"
#if __DEBUG__
                                                        " DEBUG"
#endif // __DEBUG__
                                                   "</h3>"
                                                   "<p align=\"center\"><em>%s</em></p>"
                                                   "<p align=\"center\">"

                                                     "<a href=\"http://%s\">%s</a>"
                                                   "</p>"
                                                   "<p align=\"center\">%s. All rights reserved.</p>"
                                                   "<p>This program is licensed under the terms of the "
                                                        "<a href=\"http://www.nellresearch.de/licenses/gpl.txt\">GPL</a>. "
                                                        "Some parts of the source code ist licensed under the terms of the "
                                                        "<a href=\"http://www.nellresearch.de/licenses/wxl.txt\">wxWindows licence</a>. "
                                                        "Please refere to the individual source code file."
                                                   "</p>"
                                                   "<p>This program was written using:</p>"
                                                   "<ul>"
                                                     "<li><a href=\"http://www.wxwidgets.org\">%s</a> "
                                                        "published under the <a href=\"http://www.nellresearch.de/licenses/wxl.txt\">wxWindows licence</a></li>"
#if wxUSE_PDF
                                                     "<li><a href=\"http://poppler.freedesktop.org\">Poppler</a> "
                                                        "published under the <a href=\"http://www.nellresearch.de/licenses/gpl-2.0.txt\">GPLv2</a></li>"
#endif // wxUSE_PDF
#if __WXPDFDOC__
                                                     "<li><a href=\"http://wxcode.com\">wxPdfDocument</a> "
                                                        "published under the <a href=\"http://www.nellresearch.de/licenses/wxl.txt\">wxWindows licence</a></li>"
#endif // __WXPDFDOC__
                                                   "</ul>"
                                                   "<p>"
                                                        "<a href=\"http://www.nellresearch.de/licenses/gpl.txt\">"
                                                            "<img src=\"http://www.nellresearch.de/licenses/gplv3-127x51.png\">"
                                                        "</a>"
                                                        "<a href=\"http://www.wxwidgets.org\">"
                                                            "<img src=\"http://www.wxwidgets.org/logo9.jpg\">"
                                                        "</a>"
                                                   "</p>"
                                                 "</div>"
                                               "</body>"
                                             "</html>" ),
                                        cstrAppName.c_str(),
                                        cstrAppName.c_str(), cstrWebSite.c_str(), cstrVersionString.c_str(),
                                        cstrAppDescription.c_str(),
                                        cstrWebSite.c_str(), cstrWebSite.c_str(),
                                        cstrStatusCopyRight.c_str(),
                                        cstrWxVersionString.c_str()
                                      );
	poHtmlWindow->SetPage( strHtml );
	SetTitle( poHtmlWindow->GetOpenedPageTitle() );
}

wxIScanAbout::~wxIScanAbout()
{
	//(*Destroy(wxIScanAbout)
	//*)
}
