/*************************************************************
 * Name:      wxiscanframe.cpp
 * Purpose:   Code for Application Frame
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

// wxWidgets headers
#include <wx/accel.h>
#include <wx/clipbrd.h>
#include <wx/config.h>
#include <wx/fileconf.h>
#include <wx/filename.h>
#if __WXPDFDOC__
    #include <wx/pdfdoc.h>
#endif // __WXPDFDOC__
#include <wx/print.h>
#include <wx/printdlg.h>
#if __DEBUG__
    #include <wx/stdpaths.h>
#endif // __DEBUG__
#include <wx/textfile.h>

//(*InternalHeaders(wxIScanFrame)
#include <wx/string.h>
#include <wx/intl.h>
#include <wx/bitmap.h>
#include <wx/image.h>
#include <wx/artprov.h>
//*)

// Private headers
#include "wxextprocess.h"
#include "wxiscanabout.h"
#include "wxiscanapp.h"
#include "wxiscanframe.h"
#include "wxiscanimageprofile.h"
#include "wxiscanresources.h"
#include "wxscan.h"
#include "wxchainlogtextctrl.h"
#include "wxiviewprintout.h"
#include "wxoptionsdialog.h"

// Resources
//}

//(*IdInit(wxIScanFrame)
const long wxIScanFrame::ID_CANVAS = wxNewId();
const long wxIScanFrame::ID_LOGTEXTCTRL = wxNewId();
const long wxIScanFrame::ID_HSPLITTERWINDOW = wxNewId();
const long wxIScanFrame::ID_FILESCANIMAGE = wxNewId();
const long wxIScanFrame::ID_FILEOPEN = wxNewId();
const long wxIScanFrame::ID_FILESAVEAS = wxNewId();
const long wxIScanFrame::ID_FILEPRINT = wxNewId();
const long wxIScanFrame::ID_FILEPRINTPREVIEW = wxNewId();
const long wxIScanFrame::ID_FILEPRINTSETUP = wxNewId();
const long wxIScanFrame::ID_FILEQUIT = wxNewId();
const long wxIScanFrame::ID_EDITCOPY = wxNewId();
const long wxIScanFrame::ID_EDITPASTE = wxNewId();
const long wxIScanFrame::ID_EDITSETTINGS = wxNewId();
const long wxIScanFrame::ID_EDITSETTINGSFILE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTCOPYTEXT = wxNewId();
const long wxIScanFrame::ID_DOCUMENTREADTEXT = wxNewId();
const long wxIScanFrame::ID_CMDSTOP = wxNewId();
const long wxIScanFrame::ID_DOCUMENTFIRSTPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTPREVIOUSPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTNEXTPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTLASTPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTCROP = wxNewId();
const long wxIScanFrame::ID_DOCUMENTROTATE90LEFT = wxNewId();
const long wxIScanFrame::ID_DOCUMENTROTATE90RIGHT = wxNewId();
const long wxIScanFrame::ID_DOCUMENTAPPLYPROFILE = wxNewId();
const long wxIScanFrame::ID_MENUDOCUMENTPROFILES = wxNewId();
const long wxIScanFrame::ID_PDFBEGINPDF = wxNewId();
const long wxIScanFrame::ID_PDFADDPAGE = wxNewId();
const long wxIScanFrame::ID_PDFSAVEASPDF = wxNewId();
const long wxIScanFrame::ID_HELPABOUT = wxNewId();
const long wxIScanFrame::ID_STATUSBAR = wxNewId();
const long wxIScanFrame::ID_FILESCANIMAGE_TB = wxNewId();
const long wxIScanFrame::ID_FILEOPEN_TB = wxNewId();
const long wxIScanFrame::ID_FILESAVEAS_TB = wxNewId();
const long wxIScanFrame::ID_FILEPRINT_TB = wxNewId();
const long wxIScanFrame::ID_PDFBEGINPDF_TB = wxNewId();
const long wxIScanFrame::ID_PDFADDPAGE_TB = wxNewId();
const long wxIScanFrame::ID_PDFSAVEASPDF_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTFIRSTPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTPREVIOUSPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTNEXTPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTLASTPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTREADTEXT_TB = wxNewId();
const long wxIScanFrame::ID_CMDSTOP_TB = wxNewId();
const long wxIScanFrame::ID_TOOLBAR = wxNewId();
//*)
const long wxIScanFrame::ID_DOCUMENTPROFILE0= wxNewId();


//////////////////////////////////////////////////////////
// Strings and other (global) constants
//
static const wxString cstrwxFrame= wxT( "wxIScanFrame" );


//////////////////////////////////////////////////////////
// Event table
//
BEGIN_EVENT_TABLE( wxIScanFrame, wxFrame )
    //(*EventTable(wxIScanFrame)
    //*)
    // File history handling.
    EVT_MENU_RANGE( wxID_FILE1, wxID_FILE9, wxIScanFrame::OnMRUFiles )
    EVT_MENU_RANGE( ID_DOCUMENTPROFILE0, ID_DOCUMENTPROFILE0 + 999, wxIScanFrame::OnMRUFiles )
    // File drop event
    EVT_EXTFILEDROP( wxIScanFrame::OnFileDrop )
    // Handle canvas size changes.
    EVT_IVIEWCANVASSIZE( wxIScanFrame::OnCanvasSize )
    // Handle process events.
    EVT_END_PROCESS( wxISCAN_PROCESS, wxIScanFrame::OnProcessTerminated )
END_EVENT_TABLE()

wxIScanFrame::wxIScanFrame( wxWindow* parent, wxWindowID id )
  :
  m_bUILocked( false ), m_bPortrait( true ), m_nProfileIndex( 0 ),
#if __WXPDFDOC__
  m_poPdfDoc( NULL ),  m_nPdfPages( 0 ), m_bPdfDirty( false ),
#endif
 m_poRunningProcess( NULL )
{
    //(*Initialize(wxIScanFrame)
    wxToolBarToolBase* poCmdStopTb;
    wxMenu* poMenuHelp;
    wxMenuItem* poMenuEditSettings;
    wxMenuItem* poMenuSavePdf;
    wxMenuItem* poDocumentRotate90Left;
    wxMenuItem* poMenuFileOpen;
    wxMenuItem* poMenuFileSaveAs;
    wxMenuItem* poHelpAbout;
    wxMenuItem* poMenuEditSettingsFile;
    wxMenuItem* poMenuCmdStop;
    wxMenuItem* poFileQuit;
    wxMenuItem* poMenuPdfBeginPdf;
    wxMenuItem* poMenuFileScanImage;
    wxMenuItem* poMenuDocumentPreviousPage;
    wxMenuItem* poMenuFilePrint;
    wxMenuItem* poOnDocumentCrop;
    wxMenu* poMenuDocument;
    wxMenuItem* poMenuFilePrintPreview;
    wxMenuItem* poMenuDocumentLastPage;
    wxMenuBar* m_poMenuBar;
    wxMenuItem* poMenuDocumentReadText;
    wxMenuItem* poMenuPdfAddPage;
    wxToolBar* poToolBar;
    wxMenu* poMenuDocumentProfiles;
    wxMenuItem* poDocumentApplyProfile;
    wxMenuItem* poMenuDocumentCopyText;
    wxMenuItem* poEditCopy;
    wxMenu* poMenuFile;
    wxMenu* poMenuPdf;
    wxMenuItem* poMenuFilePrintSetup;
    wxMenuItem* poDocumentRotate90Right;
    wxMenuItem* poMenuDocumentFirstPage;
    wxMenuItem* poMenuDocumentNextPage;

    Create(parent, wxID_ANY, _("wxIScan"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE, _T("wxID_ANY"));
    m_poHSplitterWindow = new wxSplitterWindow(this, ID_HSPLITTERWINDOW, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxSP_LIVE_UPDATE, _T("ID_HSPLITTERWINDOW"));
    m_poHSplitterWindow->SetMinSize(wxSize(10,10));
    m_poHSplitterWindow->SetMinimumPaneSize(10);
    m_poCanvas = new wxIViewCanvas(m_poHSplitterWindow, ID_CANVAS, wxDefaultPosition, wxDefaultSize, wxHSCROLL|wxVSCROLL, _T("ID_CANVAS"));
    m_poLogTextCtrl = new wxTextCtrl(m_poHSplitterWindow, ID_LOGTEXTCTRL, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH|wxTE_AUTO_URL|wxTE_DONTWRAP, wxDefaultValidator, _T("ID_LOGTEXTCTRL"));
    m_poHSplitterWindow->SplitHorizontally(m_poCanvas, m_poLogTextCtrl);
    m_poHSplitterWindow->SetSashPosition(250);
    m_poMenuBar = new wxMenuBar();
    poMenuFile = new wxMenu();
    poMenuFileScanImage = new wxMenuItem(poMenuFile, ID_FILESCANIMAGE, _("Scan &Image\tCtrl+I"), _("Scan an image"), wxITEM_NORMAL);
    poMenuFile->Append(poMenuFileScanImage);
    poMenuFileScanImage= poMenuFile->Remove( ID_FILESCANIMAGE );
    poMenuFileScanImage->SetBitmap( filescanimagebmp_xpm );
    poMenuFile->Append( poMenuFileScanImage );
    poMenuFile->AppendSeparator();
    poMenuFileOpen = new wxMenuItem(poMenuFile, ID_FILEOPEN, _("&Open...\tCtrl+O"), _("Open an image file."), wxITEM_NORMAL);
    poMenuFileOpen->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")),wxART_MENU));
    poMenuFile->Append(poMenuFileOpen);
    poMenuFileSaveAs = new wxMenuItem(poMenuFile, ID_FILESAVEAS, _("Save &as...\tCtrl+S"), _("Save scanned image to file."), wxITEM_NORMAL);
    poMenuFileSaveAs->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_MENU));
    poMenuFile->Append(poMenuFileSaveAs);
    poMenuFileSaveAs->Enable(false);
    poMenuFile->AppendSeparator();
    poMenuFilePrint = new wxMenuItem(poMenuFile, ID_FILEPRINT, _("&Print...\tCtrl+P"), _("Print the current image."), wxITEM_NORMAL);
    poMenuFilePrint->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_PRINT")),wxART_MENU));
    poMenuFile->Append(poMenuFilePrint);
    poMenuFilePrint->Enable(false);
    poMenuFilePrintPreview = new wxMenuItem(poMenuFile, ID_FILEPRINTPREVIEW, _("Print pre&view\tCtrl+Alt+P"), _("Show a print preview of the current image."), wxITEM_NORMAL);
    poMenuFile->Append(poMenuFilePrintPreview);
    poMenuFilePrintPreview->Enable(false);
    poMenuFilePrintPreview= poMenuFile->Remove( ID_FILEPRINTPREVIEW );
    poMenuFilePrintPreview->SetBitmap( fileprintpreviewbmp_xpm );
    poMenuFile->Append( poMenuFilePrintPreview );
    poMenuFilePrintSetup = new wxMenuItem(poMenuFile, ID_FILEPRINTSETUP, _("Page setup"), _("Show page setup dialog."), wxITEM_NORMAL);
    poMenuFile->Append(poMenuFilePrintSetup);
    poMenuFilePrintSetup= poMenuFile->Remove( ID_FILEPRINTSETUP );
    poMenuFilePrintSetup->SetBitmap( fileprintsetupbmp_xpm );
    poMenuFile->Append( poMenuFilePrintSetup );
    poMenuFile->AppendSeparator();
    poFileQuit = new wxMenuItem(poMenuFile, ID_FILEQUIT, _("&Quit\tCtrl+Q"), _("Quit the application"), wxITEM_NORMAL);
    poFileQuit->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_QUIT")),wxART_MENU));
    poMenuFile->Append(poFileQuit);
    m_poMenuBar->Append(poMenuFile, _("&File"));
    Menu1 = new wxMenu();
    poEditCopy = new wxMenuItem(Menu1, ID_EDITCOPY, _("&Copy\tCtrl+C"), _("Copy image to clipboard"), wxITEM_NORMAL);
    poEditCopy->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_COPY")),wxART_MENU));
    Menu1->Append(poEditCopy);
    poEditCopy->Enable(false);
    MenuItem1 = new wxMenuItem(Menu1, ID_EDITPASTE, _("&Paste\tCtrl-V"), _("Inserts the content of the clipboard."), wxITEM_NORMAL);
    Menu1->Append(MenuItem1);
    Menu1->AppendSeparator();
    poMenuEditSettings = new wxMenuItem(Menu1, ID_EDITSETTINGS, _("&Settings...\tCtrl-Alt-O"), _("Edit (most) settings in a dialog window."), wxITEM_NORMAL);
    poMenuEditSettings->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_EXECUTABLE_FILE")),wxART_MENU));
    Menu1->Append(poMenuEditSettings);
    poMenuEditSettingsFile = new wxMenuItem(Menu1, ID_EDITSETTINGSFILE, _("Edit Settings &file"), _("Edit settings file in a text editor."), wxITEM_NORMAL);
    poMenuEditSettingsFile->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_HELP_SETTINGS")),wxART_MENU));
    Menu1->Append(poMenuEditSettingsFile);
    m_poMenuBar->Append(Menu1, _("&Edit"));
    poMenuDocument = new wxMenu();
    poMenuDocumentCopyText = new wxMenuItem(poMenuDocument, ID_DOCUMENTCOPYTEXT, _("Copy &text\tCtrl-Alt-C"), _("Copies via OCR detected text to the clipboard."), wxITEM_NORMAL);
    poMenuDocumentCopyText->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_COPY")),wxART_MENU));
    poMenuDocument->Append(poMenuDocumentCopyText);
    poMenuDocumentCopyText->Enable(false);
    poMenuDocumentReadText = new wxMenuItem(poMenuDocument, ID_DOCUMENTREADTEXT, _("&Read out document\tCtrl+Alt+R"), _("Reads the document through TTS."), wxITEM_NORMAL);
    poMenuDocument->Append(poMenuDocumentReadText);
    poMenuDocumentReadText->Enable(false);
    poMenuDocumentReadText= poMenuDocument->Remove( ID_DOCUMENTREADTEXT );
    poMenuDocumentReadText->SetBitmap( documentreadtext_xpm );
    poMenuDocument->Append( poMenuDocumentReadText );
    poMenuCmdStop = new wxMenuItem(poMenuDocument, ID_CMDSTOP, _("&Stop\tCtrl-Alt-E"), _("Stop background action."), wxITEM_NORMAL);
    poMenuDocument->Append(poMenuCmdStop);
    poMenuCmdStop->Enable(false);
    poMenuCmdStop= poMenuDocument->Remove( ID_CMDSTOP );
    poMenuCmdStop->SetBitmap( cmdstop_xpm );
    poMenuDocument->Append( poMenuCmdStop );
    poMenuDocument->AppendSeparator();
    poMenuDocumentFirstPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTFIRSTPAGE, _("&First page\tCtrl+Home"), _("Go to first document page."), wxITEM_NORMAL);
    poMenuDocument->Append(poMenuDocumentFirstPage);
    poMenuDocumentFirstPage->Enable(false);
    poMenuDocumentFirstPage= poMenuDocument->Remove( ID_DOCUMENTFIRSTPAGE );
    poMenuDocumentFirstPage->SetBitmap( documentfirstpage_xpm );
    poMenuDocument->Append( poMenuDocumentFirstPage );
    poMenuDocumentPreviousPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTPREVIOUSPAGE, _("&Previous page\tCtrl+PageUp"), _("Go to previous document page."), wxITEM_NORMAL);
    poMenuDocument->Append(poMenuDocumentPreviousPage);
    poMenuDocumentPreviousPage->Enable(false);
    poMenuDocumentPreviousPage= poMenuDocument->Remove( ID_DOCUMENTPREVIOUSPAGE );
    poMenuDocumentPreviousPage->SetBitmap( documentpreviouspage_xpm );
    poMenuDocument->Append( poMenuDocumentPreviousPage );
    poMenuDocumentNextPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTNEXTPAGE, _("&Next page\tCtrl+PageDown"), _("Go to next document page."), wxITEM_NORMAL);
    poMenuDocument->Append(poMenuDocumentNextPage);
    poMenuDocumentNextPage->Enable(false);
    poMenuDocumentNextPage= poMenuDocument->Remove( ID_DOCUMENTNEXTPAGE );
    poMenuDocumentNextPage->SetBitmap( documentnextpage_xpm );
    poMenuDocument->Append( poMenuDocumentNextPage );
    poMenuDocumentLastPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTLASTPAGE, _("&Last page\tCtrl+End"), _("Go to last document page."), wxITEM_NORMAL);
    poMenuDocument->Append(poMenuDocumentLastPage);
    poMenuDocumentLastPage->Enable(false);
    poMenuDocumentLastPage= poMenuDocument->Remove( ID_DOCUMENTLASTPAGE );
    poMenuDocumentLastPage->SetBitmap( documentlastpage_xpm );
    poMenuDocument->Append( poMenuDocumentLastPage );
    poMenuDocument->AppendSeparator();
    poOnDocumentCrop = new wxMenuItem(poMenuDocument, ID_DOCUMENTCROP, _("&Crop selection\tCtrl+Y"), _("Crop the image along selection rectangle."), wxITEM_NORMAL);
    poMenuDocument->Append(poOnDocumentCrop);
    poOnDocumentCrop->Enable(false);
    poDocumentRotate90Left = new wxMenuItem(poMenuDocument, ID_DOCUMENTROTATE90LEFT, _("Rotate 90 degress &left\tL"), _("Rotate 90 degress left"), wxITEM_NORMAL);
    poDocumentRotate90Left->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_UNDO")),wxART_MENU));
    poMenuDocument->Append(poDocumentRotate90Left);
    poDocumentRotate90Left->Enable(false);
    poDocumentRotate90Right = new wxMenuItem(poMenuDocument, ID_DOCUMENTROTATE90RIGHT, _("Rotate 90 degress &right\tR"), _("Rotate 90 degress right"), wxITEM_NORMAL);
    poDocumentRotate90Right->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_REDO")),wxART_MENU));
    poMenuDocument->Append(poDocumentRotate90Right);
    poDocumentRotate90Right->Enable(false);
    poMenuDocument->AppendSeparator();
    poDocumentApplyProfile = new wxMenuItem(poMenuDocument, ID_DOCUMENTAPPLYPROFILE, _("&Apply profile\tp"), _("Apply the current selected image profile to the image."), wxITEM_NORMAL);
    poMenuDocument->Append(poDocumentApplyProfile);
    poDocumentApplyProfile->Enable(false);
    poMenuDocumentProfiles = new wxMenu();
    poMenuDocumentProfiles->AppendSeparator();
    poMenuDocument->Append(ID_MENUDOCUMENTPROFILES, _("&Profiles"), poMenuDocumentProfiles, wxEmptyString);
    m_poMenuBar->Append(poMenuDocument, _("&Document"));
    poMenuPdf = new wxMenu();
    poMenuPdfBeginPdf = new wxMenuItem(poMenuPdf, ID_PDFBEGINPDF, _("&Begin PDF\tCtrl+Alt+N"), _("Start a new PDF document."), wxITEM_NORMAL);
    poMenuPdfBeginPdf->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_NEW")),wxART_MENU));
    poMenuPdf->Append(poMenuPdfBeginPdf);
    poMenuPdfBeginPdf->Enable(false);
    poMenuPdfAddPage = new wxMenuItem(poMenuPdf, ID_PDFADDPAGE, _("&Add Page\tCtrl++"), _("Add image as new page to the PDF document."), wxITEM_NORMAL);
    poMenuPdfAddPage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_ADD_BOOKMARK")),wxART_MENU));
    poMenuPdf->Append(poMenuPdfAddPage);
    poMenuPdfAddPage->Enable(false);
    poMenuPdf->AppendSeparator();
    poMenuSavePdf = new wxMenuItem(poMenuPdf, ID_PDFSAVEASPDF, _("Save &PDF...\tCtrl+Alt+S"), _("Save image as PDF file."), wxITEM_NORMAL);
    poMenuSavePdf->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE")),wxART_MENU));
    poMenuPdf->Append(poMenuSavePdf);
    poMenuSavePdf->Enable(false);
    m_poMenuBar->Append(poMenuPdf, _("&PDF"));
    poMenuHelp = new wxMenu();
    poHelpAbout = new wxMenuItem(poMenuHelp, ID_HELPABOUT, _("&About\tCtrl-Alt-I"), _("Show info about this application"), wxITEM_NORMAL);
    poHelpAbout->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_INFORMATION")),wxART_MENU));
    poMenuHelp->Append(poHelpAbout);
    m_poMenuBar->Append(poMenuHelp, _("&Help"));
    SetMenuBar(m_poMenuBar);
    m_poStatusBar = new wxStatusBar(this, ID_STATUSBAR, 0, _T("ID_STATUSBAR"));
    int __wxStatusBarWidths_1[4] = { -3, 80, 80, -2 };
    int __wxStatusBarStyles_1[4] = { wxSB_NORMAL, wxSB_NORMAL, wxSB_NORMAL, wxSB_NORMAL };
    m_poStatusBar->SetFieldsCount(4,__wxStatusBarWidths_1);
    m_poStatusBar->SetStatusStyles(4,__wxStatusBarStyles_1);
    SetStatusBar(m_poStatusBar);
    poToolBar = new wxToolBar(this, ID_TOOLBAR, wxDefaultPosition, wxDefaultSize, wxTB_VERTICAL|wxNO_BORDER, _T("ID_TOOLBAR"));
    poToolBar->SetToolBitmapSize(wxSize(16,16));
    ToolBarItem1 = poToolBar->AddTool(ID_FILESCANIMAGE_TB, _("Scan"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_MISSING_IMAGE")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Scan"), _("Scan an image"));
    ToolBarItem2 = poToolBar->AddTool(ID_FILEOPEN_TB, _("Open"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Open file"), _("Open an image file."));
    ToolBarItem3 = poToolBar->AddTool(ID_FILESAVEAS_TB, _("Save as"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Save as"), _("Save scanned image to file."));
    ToolBarItem4 = poToolBar->AddTool(ID_FILEPRINT_TB, _("Print"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_PRINT")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Print"), _("Print the current image."));
    poToolBar->AddSeparator();
    ToolBarItem5 = poToolBar->AddTool(ID_PDFBEGINPDF_TB, _("Begin PDF"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_NEW")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Begin PDF"), _("Start a new PDF document."));
    ToolBarItem6 = poToolBar->AddTool(ID_PDFADDPAGE_TB, _("Add Page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_ADD_BOOKMARK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Add PDF Page"), _("Add image as new page to the PDF document."));
    ToolBarItem7 = poToolBar->AddTool(ID_PDFSAVEASPDF_TB, _("Save PDF"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Save PDF"), _("Save image as PDF file."));
    poToolBar->AddSeparator();
    ToolBarItem8 = poToolBar->AddTool(ID_DOCUMENTFIRSTPAGE_TB, _("First page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("First page"), _("Go To first document page."));
    ToolBarItem9 = poToolBar->AddTool(ID_DOCUMENTPREVIOUSPAGE_TB, _("Previous page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Previous page"), _("Go to previous document page."));
    ToolBarItem10 = poToolBar->AddTool(ID_DOCUMENTNEXTPAGE_TB, _("Next page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Next page"), _("Go to next document page."));
    ToolBarItem11 = poToolBar->AddTool(ID_DOCUMENTLASTPAGE_TB, _("Last page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Last page"), _("Go to last document page."));
    poToolBar->AddSeparator();
    ToolBarItem12 = poToolBar->AddTool(ID_DOCUMENTREADTEXT_TB, _("Read out"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_HELP_BOOK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Read out document via TTS."), _("Read out document via text-to-speach system."));
    poCmdStopTb = poToolBar->AddTool(ID_CMDSTOP_TB, _("Stop"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_ERROR")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Stop background action."), _("Stop background action."));
    poToolBar->Realize();
    SetToolBar(poToolBar);

    Connect(ID_FILESCANIMAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileScanImage);
    Connect(ID_FILEOPEN,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileOpen);
    Connect(ID_FILESAVEAS,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileSaveAs);
    Connect(ID_FILEPRINT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrint);
    Connect(ID_FILEPRINTPREVIEW,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrintPreview);
    Connect(ID_FILEPRINTSETUP,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrintSetup);
    Connect(ID_FILEQUIT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileQuit);
    Connect(ID_EDITCOPY,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnEditCopy);
    Connect(ID_EDITPASTE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnEditPaste);
    Connect(ID_EDITSETTINGS,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnEditSettings);
    Connect(ID_EDITSETTINGSFILE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnEditSettingsFile);
    Connect(ID_DOCUMENTCOPYTEXT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentCopyText);
    Connect(ID_DOCUMENTREADTEXT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentReadText);
    Connect(ID_CMDSTOP,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnCmdStop);
    Connect(ID_DOCUMENTFIRSTPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentFirstPage);
    Connect(ID_DOCUMENTPREVIOUSPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentPreviousPage);
    Connect(ID_DOCUMENTNEXTPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentNextPage);
    Connect(ID_DOCUMENTLASTPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentLastPage);
    Connect(ID_DOCUMENTCROP,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentCrop);
    Connect(ID_DOCUMENTROTATE90LEFT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentRotate90Left);
    Connect(ID_DOCUMENTROTATE90RIGHT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentRotate90Right);
    Connect(ID_DOCUMENTAPPLYPROFILE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentApplyProfile);
    Connect(ID_PDFBEGINPDF,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnPdfBeginPdf);
    Connect(ID_PDFADDPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnPdfAddPage);
    Connect(ID_PDFSAVEASPDF,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnPdfSavePdf);
    Connect(ID_HELPABOUT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnHelpAbout);
    Connect(ID_FILESCANIMAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFileScanImage);
    Connect(ID_FILEOPEN_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFileOpen);
    Connect(ID_FILESAVEAS_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFileSaveAs);
    Connect(ID_FILEPRINT_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrint);
    Connect(ID_PDFBEGINPDF_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnPdfBeginPdf);
    Connect(ID_PDFADDPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnPdfAddPage);
    Connect(ID_PDFSAVEASPDF_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnPdfSavePdf);
    Connect(ID_DOCUMENTFIRSTPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentFirstPage);
    Connect(ID_DOCUMENTPREVIOUSPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentPreviousPage);
    Connect(ID_DOCUMENTNEXTPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentNextPage);
    Connect(ID_DOCUMENTLASTPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentLastPage);
    Connect(ID_DOCUMENTREADTEXT_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentReadText);
    Connect(ID_CMDSTOP_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnCmdStop);
    Connect(wxID_ANY,wxEVT_CLOSE_WINDOW,(wxObjectEventFunction)&wxIScanFrame::OnClose);
    //*)

    // Custom tool bar icons.
    GetToolBar()->SetToolNormalBitmap( ID_CMDSTOP_TB,              wxBitmap( cmdstop_xpm ) );
    GetToolBar()->SetToolNormalBitmap( ID_DOCUMENTREADTEXT_TB,     wxBitmap( documentreadtext_xpm ) );
    GetToolBar()->SetToolNormalBitmap( ID_DOCUMENTFIRSTPAGE_TB,    wxBitmap( documentfirstpage_xpm ) );
    GetToolBar()->SetToolNormalBitmap( ID_DOCUMENTPREVIOUSPAGE_TB, wxBitmap( documentpreviouspage_xpm ) );
    GetToolBar()->SetToolNormalBitmap( ID_DOCUMENTNEXTPAGE_TB,     wxBitmap( documentnextpage_xpm ) );
    GetToolBar()->SetToolNormalBitmap( ID_DOCUMENTLASTPAGE_TB,     wxBitmap( documentlastpage_xpm ) );
    GetToolBar()->SetToolNormalBitmap( ID_FILESCANIMAGE_TB ,       wxBitmap( filescanimagebmp_xpm ) );

//    // Custom key accelerators.
//    wxAcceleratorEntry aoAcceleratorEntries[1];
//    aoAcceleratorEntries[0].Set( wxACCEL_NORMAL, 'a', ID_CMDSTOP_TB );
//    wxAcceleratorTable oAcceleratorTable( WXSIZEOF( aoAcceleratorEntries ),
//                                          aoAcceleratorEntries );
//    SetAcceleratorTable( oAcceleratorTable );

    // Custom Initializations:
    //
    m_poCanvas->SetOptions(   wxIViewCanvas::wxICANVAS_MOUSE_SCROLL
                            | wxIViewCanvas::wxICANVAS_MOUSE_SELECTION );

    // Reset the image count, the page number and the image filename.
    ResetFileName();

#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    // Initialize a new pdf document object.
    if( !CreateNewPdfDocument() )
    {
        ::wxLogFatalError( _( "Failed to create wxPdfDocument object." ) );
    }
#endif

    // Complete status bar information.
    SetStatusText( cstrStatusCopyRight, 3 );

    // Give the window an icon.
    SetIcon( wxiscanico_xpm );

    // Restore (most) settings.
    RestoreSettings();

    // Restore some additional settings (which should not be initialized twice).
    wxConfigBase *poConfig = wxConfigBase::Get();
    poConfig->SetRecordDefaults();
    poConfig->SetPath( wxT( "/" )  );
    poConfig->SetPath( cstrwxFrame );

    // Set style attributes to log text control:
    //
    // Read the font size from config file, ...
    int nFontSize= poConfig->Read( wxT( "FontSize" ), 8 );

    // ... get default text style parameters, and ...
    wxTextAttr oStyle( m_poLogTextCtrl->GetDefaultStyle() );

    // ... set some styling options (e. g. font).
    wxFont oFont( nFontSize, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL );

    oStyle.SetFont( oFont );
    m_poLogTextCtrl->SetDefaultStyle( oStyle );

    // Redirect logging to text control.
    int nMaxLogLevel= poConfig->Read( wxT( "MaxLogLevel" ), 2 );
    new wxChainLogTextCtrl( m_poLogTextCtrl, nMaxLogLevel );

#if __DEBUG__
    ::wxLogMessage( wxT( "%s %s" ), cstrAppName.c_str(), cstrVersionString.c_str() );
    ::wxLogMessage( wxT( "%s" ), cstrAppDescription.c_str() );
    ::wxLogMessage( wxT( "%s" ), cstrStatusCopyRight.c_str() );
    ::wxLogMessage( wxT( "----------------------------------------------------------" ) );
#endif

    // Restore profile Settings
    int nProfileIndex= poConfig->Read( wxT( "ProfileIndex" ), (long)0 );

    for( int i= 0; i < 999; i++ )
    {
        wxString strMenuItemText;

        if( !poConfig->Exists( wxString::Format( wxT( "/Profile/%d" ), i ) ) )
        {
            break;
        }

        wxIScanImageProfile oProfile( i, m_nResolution );

        if( i <= 9 )
        {
            strMenuItemText= wxString::Format( wxT( "&%d " ), i ) + oProfile.GetName() + wxString::Format( wxT( "\tCtrl+%d" ), i );
        }
        else
        {
            strMenuItemText= wxString::Format( wxT( "%d " ), i ) + oProfile.GetName();
        }

        int nMenuID= ID_DOCUMENTPROFILE0 + i;

        poMenuDocumentProfiles->AppendCheckItem( nMenuID, strMenuItemText );
        Connect( nMenuID, wxEVT_COMMAND_MENU_SELECTED, (wxObjectEventFunction)&wxIScanFrame::OnDocumentProfilesChooseProfile );
    }
    SetProfile( nProfileIndex );

    // Reset file history.
    poConfig->SetPath( wxT( "/Files" ) );
    m_oFileHistory.Load( *poConfig );

    // Delete file history entries of files that do not exist.
    for( int i= m_oFileHistory.GetCount() - 1; i >= 0; i-- )
    {
        wxString strFileName= m_oFileHistory.GetHistoryFile( i );

        if( !::wxFileExists( strFileName ) )
        {
            m_oFileHistory.RemoveFileFromHistory( i );
        }
    }

    // Add file history to the 'files' menu.
    m_oFileHistory.AddFilesToMenu( poMenuFile );
    m_oFileHistory.UseMenu( poMenuFile );

    // Set drop target.
    SetDropTarget( new wxExtFileDropTarget( this ) );

    // Switch (mostly disable) controls
    EnableControls();
}

wxIScanFrame::~wxIScanFrame()
{
    // Terminate a possibly running concurrenct process.
    if( m_poRunningProcess )
    {
        m_poRunningProcess->Kill( wxSIGTERM, wxKILL_CHILDREN );
        delete m_poRunningProcess;
        m_poRunningProcess= NULL;
    }

#if __WXPDFDOC__
    // Destroy pending PDF documents.
    if( m_poPdfDoc )
    {
        delete m_poPdfDoc;
    }
#endif // __WXPDFDOC__

#if __DEBUG__
    // Save the log to a text file.
    wxFileName oFileName( wxStandardPaths::Get().GetDocumentsDir(), ::wxGetApp().GetAppName(), wxT( "log" ) );

    if( m_poLogTextCtrl )
    {
        m_poLogTextCtrl->SaveFile( oFileName.GetFullPath() );
    }
#endif // __DEBUG__

    // Save configuration data.
    SaveSettings();

    //(*Destroy(wxIScanFrame)
    //*)
}

void wxIScanFrame::OnClose( wxCloseEvent& oEvent )
{
    // Check the SQL edit control if the content is "dirty" and veto closing
    // (if it is not forbidden).
    if( CanClose( oEvent.CanVeto() ) )
    {
        this->Destroy();
    }
    else
    {
        if( oEvent.CanVeto() )
        {
            oEvent.Veto();
        }
        else
        {
            this->Destroy();
        }
    }
}

void wxIScanFrame::OnFileDrop( wxExtFileDropEvent &oEvent )
{
    // Open file.
    FileOpen( oEvent.GetFileName() );
}

void wxIScanFrame::OnFileScanImage( wxCommandEvent & WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Scan image.
    ScanImage();

    // Enable (additional) menu options after the 1st scan.
    EnableControls();
}

void wxIScanFrame::OnFileOpen( wxCommandEvent& event )
{
    // Open an "Open file" dialog an let the user choose a filename.
    wxFileDialog oDlg( this,
                       _( "Open file" ),
                       wxT( "" ),
                       wxT( "" ),
                       ::wxGetApp().GetImageFilter(),
                       wxOPEN | wxFILE_MUST_EXIST | wxCHANGE_DIR );

    if( oDlg.ShowModal() == wxID_OK )
    {
        // Open file.
        FileOpen( oDlg.GetPath() );
    }
}

void wxIScanFrame::OnFileSaveAs( wxCommandEvent& event )
{
    // Open a "Save file" dialog and let the user choose a filename.
    wxFileDialog oDlg( this,
                       _( "Save as" ),
                       wxT( "" ),
                       m_strImageFileName,
                       ::wxGetApp().GetImageFilter(),
                       wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );

    if( oDlg.ShowModal() == wxID_OK )
    {
        // Show hourglass cursor.
        wxBusyCursor oWait;

        // Save file.
        m_oImage.SaveFile( oDlg.GetPath() );

        // Save the filename.
        m_strImageFileName= oDlg.GetPath();
        SetTitle( m_strImageFileName + wxT( " - " ) + cstrAppName );
    }
}

void wxIScanFrame::OnFilePrint( wxCommandEvent & WXUNUSED( oEvent ) )
{
    AdjustOrientation();

    // Data object representing the print dialog's values.
    wxPrintDialogData oPrintDialogData( m_oPrintData );

    // Set the printer dialog's maximal page.
    oPrintDialogData.SetMaxPage( m_nImageCount );

    // Printer object.
    wxPrinter oPrinter( &oPrintDialogData );

    // ...
    wxIViewPrintout oPrintout( this, m_strImageFileName, false, m_nResolution, m_nResolution );

    // Create cancel dialog.
    oPrinter.CreateAbortWindow( this, &oPrintout );

    // Print.
    if( oPrinter.Print( this, &oPrintout, true ) )
    {
        m_oPrintData= oPrinter.GetPrintDialogData().GetPrintData();
    }
    else
    {
        if( wxPrinter::GetLastError() == wxPRINTER_ERROR )
        {
            ::wxLogError( _( "An error occured while printing." ) );
        }
    }

    // Destory the print image (for to save MBs memory).
    m_oPrintImage.Destroy();
}

void wxIScanFrame::OnFilePrintPreview( wxCommandEvent & WXUNUSED( oEvent ) )
{
    AdjustOrientation();

    wxPrintDialogData oPrintDialogData( m_oPrintData );

    // Pass two printout objects: for preview, and possible printing.
    wxPrintPreview *poPreview = new wxPrintPreview(
        new wxIViewPrintout( this, m_strImageFileName, false, m_nResolution, m_nResolution ),
        new wxIViewPrintout( this, m_strImageFileName, false, m_nResolution, m_nResolution ),
        &oPrintDialogData );

    if ( !poPreview || !poPreview->Ok() )
    {
        if( poPreview )
        {
            delete poPreview;
        }
        ::wxLogError( _( "An error occured while creating print preview object.") );
        return;
    }

    // Set the zoom factor to 100%.
    poPreview->SetZoom( 100 );

    int x, y, w, h;

	// Get the frame window position and size and create a print preview
	// window with the same postion and size.
    GetSize( &w, &h );
    GetPosition( &x, &y );
    wxIViewPreviewFrame *poPreviewFrame= new wxIViewPreviewFrame(
                                          poPreview,
                                          this,
                                          _( "Print Preview" ),
                                          wxPoint( x, y ),
                                          wxSize( w, h ) );
    if( !poPreviewFrame )
    {
        delete poPreview;
        ::wxLogError( _( "An error occured while creating print preview window.") );
        return;
    }
    poPreviewFrame->Initialize();
    poPreviewFrame->Show();

    // Destory the print image (for to save MBs memory).
    m_oPrintImage.Destroy();
}

void wxIScanFrame::OnFilePrintSetup( wxCommandEvent & WXUNUSED( oEvent ) )
{
    AdjustOrientation();

    wxPageSetupDialogData oPageSetupData( m_oPrintData );

    oPageSetupData.EnableOrientation( false );

    wxPageSetupDialog oPageSetupDialog( this, &oPageSetupData );

    oPageSetupDialog.ShowModal();
    m_oPrintData= oPageSetupDialog.GetPageSetupData().GetPrintData();
}

void wxIScanFrame::OnFileQuit( wxCommandEvent& event )
{
    Close();
}

void wxIScanFrame::OnMRUFiles( wxCommandEvent &oEvent )
{
	// Get the filename from history menu index,...
    wxString strFilename = m_oFileHistory.GetHistoryFile( oEvent.GetId() - wxID_FILE1 );

	// ... check if it is valid, and...
    if( !strFilename.IsEmpty() )
    {
		// ... open the image file again.
        FileOpen( strFilename );
    }
}

void wxIScanFrame::OnPdfBeginPdf( wxCommandEvent& event )
{
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    if( !CanClose() )
    {
        return;
    }

    if( !CreateNewPdfDocument() )
    {
        return;
    }

    // Enable (additional) menu options.
    EnableControls();
#else
    ::wxLogError( _( "PDF support is not enabled." ) );
#endif
}

void wxIScanFrame::OnPdfAddPage( wxCommandEvent& event )
{
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    bool bAddAllPages= false;

    // ...
    if( ( m_nImageCount > 1 ) && ( m_nPage == 0 ) )
    {

        switch( ::wxMessageBox( _( "This is a multipage image. Do you want to add all image pages to the PDF file?" ),
                                ::wxGetApp().GetAppName(),
                                wxYES_DEFAULT | wxNO | wxCANCEL | wxICON_QUESTION  ) )
        {
            case wxYES    : bAddAllPages= true;
                            break;
            case wxCANCEL : return;
        }
    }

    // Show busy cursor (until the automatic variable is destroyed).
    wxBusyCursor oWait;

    // Add page to PDF document.
    AddPdfPage();

    if( bAddAllPages )
    {
        while( m_nPage < ( m_nImageCount - 1 ) )
        {
            // Load next image page.
            OnDocumentNextPage( event );

            // Add page to PDF document.
            AddPdfPage();
        }
    }

    // Enable (additional) menu options.
    EnableControls();
#else
    ::wxLogError( _( "PDF support is not enabled." ) );
#endif
}

void wxIScanFrame::OnPdfSavePdf( wxCommandEvent& event )
{
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    if( !m_poPdfDoc )
    {
        ::wxLogError( _( "wxPdfDocument is NULL" ) );
        return;
    }

    // Open a "Save file" dialog and let the user choose a filename.
    wxFileDialog oDlg( this,
                       _( "Save as" ),
                       wxT( "" ),
                       _( "unknown.pdf" ),
                       _( "PDF file" ) + wxString( wxT( " (*.pdf)|*.pdf|" ) )
                       + _( "All files" ) + wxString( wxT( "|*.*" ) ),
                       wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );

    if( oDlg.ShowModal() == wxID_OK )
    {
        // Show busy cursor (until the automatic variable is destroyed).
        wxBusyCursor oWait;

        // Save file.
        m_poPdfDoc->SaveAsFile( oDlg.GetPath() );

        // Mark PDF document as 'clean'.
        m_bPdfDirty= false;

        // Create a new PDF document.
        OnPdfBeginPdf( event );
    }
#else
    ::wxLogError( _( "PDF support is not enabled." ) );
#endif
}

void wxIScanFrame::OnEditCopy( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // ...
    CopyImage2Clipboard( GetImageSelection() );
}

void wxIScanFrame::OnEditPaste(wxCommandEvent& event)
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    wxBitmapDataObject oBitmapData;

    // Open the clipboard.
    //
    // Note: There is a global object ::wxTheClipboard.
    //
    if( !wxTheClipboard->Open() )
    {
        ::wxLogError( wxString( wxT( "wxIScanFrame::CopyImage2Clipboard -- " ) )
              + _( "Failed to open clipboard." ) );
    }

    // If there is bitmap data in the clipboard get it.
    if( wxTheClipboard->GetData( oBitmapData ) )
    {
        // Get the image from the bitmap data in the clipboard.
        wxBitmap oBitmap= oBitmapData.GetBitmap();

        m_oImage= oBitmap.ConvertToImage();
        SetBitmap();
        m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );
        EnableControls();
    }

    // Close the clipboard.
    wxTheClipboard->Close();

    m_nImageCount= 1;
    m_nPage= 0;
    m_strImageFileName= _( "unknown.jpg" );
    SetTitle( cstrAppName );
}

void wxIScanFrame::OnEditSettings( wxCommandEvent & WXUNUSED( oEvent ) )
{
    // Get the global configuration object.
    wxConfigBase *poConfig = wxConfigBase::Get();
    poConfig->SetPath( wxT( "/Global" ) );

    // Get internationalization state directly from configuration.
    bool bI18n= (bool)poConfig->Read( wxT( "I18n" ), (long)true );

    // Create, setup and display the options dialog (as a modal dialog).
    wxOptionsDialog oDialog( this );
    int nId= WXOPTIONSDIALOGIDOPTION;

    // Create, setup and display the options dialog (as a modal dialog).
    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Use internationalization on next program start." ), &bI18n ) );
    nId++;
    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Automatically apply profile after scanning." ), &m_bAutoApplyProfile ) );
    nId++;
    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Use OCR when adding text to a PDF page." ), &m_bOCR ) );
    nId++;
    oDialog.AppendOption( new wxOptionTextBox( &oDialog, nId, _( "Mime type of image exported to a PDF page:" ), &m_strPdfBitmapMimeType, 150 ) );
    nId++;

    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Enable image convertion command." ), &m_bCmdConvertEnable ) );
    nId++;
    oDialog.AppendOption( new wxOptionTextBox( &oDialog, nId, _( "Bitmap conversion command:" ), &m_strCmdConvert ) );
    nId++;

    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Enable OCR command." ), &m_bCmdOcrEnable ) );
    nId++;
    oDialog.AppendOption( new wxOptionTextBox( &oDialog, nId, _( "OCR command:" ), &m_strCmdOcr ) );
    nId++;

    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Enable clean up command." ), &m_bCmdCleanUpEnable ) );
    nId++;
    oDialog.AppendOption( new wxOptionTextBox( &oDialog, nId, _( "Clean up command:" ), &m_strCmdCleanUp ) );
    nId++;

    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Enable TTS command." ), &m_bCmdTtsEnable ) );
    nId++;
    oDialog.AppendOption( new wxOptionTextBox( &oDialog, nId, _( "TTS command:" ), &m_strCmdTts ) );
    nId++;

    oDialog.AppendOption( new wxOptionCheckBox( &oDialog, nId, _( "Enable editor command:" ), &m_bCmdEditEnable ) );
    nId++;
    oDialog.AppendOption( new wxOptionTextBox( &oDialog, nId, _( "Editor command:" ), &m_strCmdEdit ) );
//    ...

    oDialog.SetupDialog();

    if( oDialog.ShowModal() == wxID_OK )
    {
        // Get the global configuration object and set the config path.
        //
        // NOTE: We do this because wxConfigBase::Get() returns a global object
        //       that could have been changed by the option dialog itself.
        //
        poConfig= wxConfigBase::Get();
        poConfig->SetPath( wxT( "/Global" ) );

        // Set i18n flag state directly in configuration (file or registry).
        poConfig->Write( wxT( "I18n" ), (long)bI18n );

        // ...

        // Adjust control states.
        EnableControls();
    }
}

void wxIScanFrame::OnEditSettingsFile(wxCommandEvent& event)
{
    // Save programme settings.
    SaveSettings();

    // Open settings editor.
    wxFileConfig *poConfig = (wxFileConfig *)wxConfigBase::Get();
    wxString strConfigFilename= wxFileConfig::GetLocalFileName( poConfig->GetAppName() );
    wxString strCommand= wxString::Format( m_strCmdEdit, strConfigFilename.c_str() );

    #if __DEBUG__
        ::wxLogMessage( wxT( "wxIScanFrame::OnEditSettings - strCommand == '%s'" ), strCommand.c_str() );
    #endif
    ::wxExecute( strCommand, wxEXEC_SYNC );

    // Restore settings (from file).
    RestoreSettings();
}

void wxIScanFrame::OnDocumentCopyText( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Save image in a temporary JPEG file.
    wxString strTempFileName= wxFileName::CreateTempFileName( wxT( "" ) );

    GetImageSelection().SaveFile( strTempFileName, wxBITMAP_TYPE_JPEG );

    // Do OCR (and get the file name of the generated text file).
    wxString strOcrTextFileName= OCR( strTempFileName );

    // Remove the temporary JPEG file.
    ::wxRemoveFile( strTempFileName );

    // Get the text and copy it to the clipboard.
    if( !strOcrTextFileName.IsEmpty() )
    {
        // Copy text (from text file) to clipboard.
        CopyText2Clipboard( GetTextFromFile( strOcrTextFileName ) );

        // ...
        ::wxRemoveFile( strOcrTextFileName );
    }
}

void wxIScanFrame::OnDocumentReadText( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Save image in a temporary JPEG file.
    wxString strTempFileName= wxFileName::CreateTempFileName( wxT( "" ) );

    GetImageSelection().SaveFile( strTempFileName, wxBITMAP_TYPE_JPEG );

    // Do OCR, and get the temporary text file name.
    wxString strTempTextFileName= OCR( strTempFileName );

    // Remove temporary JPEG file.
    ::wxRemoveFile( strTempFileName );

    // Read the text using a custom command.
    if( !strTempTextFileName.IsEmpty() )
    {
        wxString strCommand= wxString::Format( m_strCmdTts, strTempTextFileName.c_str() );

    #if __DEBUG__
        ::wxLogMessage( wxT( "wxIScanFrame::OnDocumentReadText - strCommand == '%s'" ), strCommand.c_str() );
    #endif

        m_poRunningProcess= new wxExtProcessDelTempFile( this, wxISCAN_PROCESS, strTempTextFileName );
        if( !m_poRunningProcess )
        {
            ::wxLogMessage( wxString( wxT( "wxIScanFrame::OnDocumentReadText - " ) )
                            + _( "Cannot create process object." ) );
        }

        int nPid= m_poRunningProcess->Execute( strCommand, wxEXEC_ASYNC | wxEXEC_MAKE_GROUP_LEADER );

        if( !nPid )
        {
            ::wxLogMessage( wxString( wxT( "wxIScanFrame::OnDocumentReadText - " ) )
                            + _( "Cannot start TTS process." ) );
        }
        EnableControls();
    }
}

void wxIScanFrame::OnDocumentFirstPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    m_nPage= 0;
    OnDocumentPageChanged();
}

void wxIScanFrame::OnDocumentPreviousPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    if( m_nPage > 0 )
    {
        m_nPage--;
        OnDocumentPageChanged();
    }
}

void wxIScanFrame::OnDocumentNextPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    if( m_nPage < ( m_nImageCount - 1 ) )
    {
        m_nPage++;
        OnDocumentPageChanged();
    }
}

void wxIScanFrame::OnDocumentLastPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    m_nPage= m_nImageCount - 1;
    OnDocumentPageChanged();
}

void wxIScanFrame::OnDocumentCrop( wxCommandEvent& WXUNUSED( oEvent ) )
{
    m_oImage= GetImageSelection();
    SetBitmap();
    ResetFileName();
    EnableControls();
}

void wxIScanFrame::OnDocumentRotate90Left( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Rotate image and change image orientation.
    m_oImage= m_oImage.Rotate90( false );
    SetBitmap();
    m_bPortrait= !m_bPortrait;
}

void wxIScanFrame::OnDocumentRotate90Right( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Rotate image and change image orientation.
    m_oImage= m_oImage.Rotate90( true );
    SetBitmap();
    m_bPortrait= !m_bPortrait;
}

void wxIScanFrame::OnDocumentApplyProfile( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Apply an image profile (crop, convert to monochrome etc.), and ...
    ApplyProfile();

    // ... reset the image on the canvas window.
    SetBitmap();
}

void wxIScanFrame::OnDocumentProfilesChooseProfile( wxCommandEvent& oEvent )
{
    SetProfile( oEvent.GetId() - ID_DOCUMENTPROFILE0 );
}

void wxIScanFrame::OnCmdStop( wxCommandEvent& oEvent )
{
    if( m_poRunningProcess )
    {
        m_poRunningProcess->Kill( wxSIGTERM, wxKILL_CHILDREN );
        m_poRunningProcess->Detach();
        m_poRunningProcess= NULL;
   }
    EnableControls();
}

void wxIScanFrame::OnHelpAbout( wxCommandEvent& event )
{
    wxIScanAbout oAboutBox( this );

    oAboutBox.ShowModal();
}

void wxIScanFrame::OnCanvasSize( wxIViewCanvasSizeEvent & WXUNUSED( oEvent ) )
{
    SetBitmap();
}

void wxIScanFrame::OnProcessTerminated( wxProcessEvent & WXUNUSED( oEvent ) )
{
    if( m_poRunningProcess )
    {
        delete m_poRunningProcess;
        m_poRunningProcess= NULL;
    }
    EnableControls();
}

bool wxIScanFrame::CanClose( bool bCanVeto )
{
#if __WXPDFDOC__
    if( IsPdfDirty() )
    {
        switch( ::wxMessageBox( _( "The changes to the current PDF document have not yet been saved. "
                                   "Do you want to save the PDF document now?" ),
                                ::wxGetApp().GetAppName(),
                                wxYES_NO | ( bCanVeto ? wxCANCEL : 0 ) | wxICON_QUESTION ) )
        {
            case wxYES: {
                            wxCommandEvent oDummyEvent( wxEVT_COMMAND_MENU_SELECTED, ID_PDFSAVEASPDF );

                            OnPdfSavePdf  ( oDummyEvent );
                        }
                        return !IsPdfDirty();
            case wxNO : return true;
            default:    return false;
        }
    }
    else
    {
        return true;
    }
#else
    return true;
#endif
}

bool wxIScanFrame::FileOpen( const wxString &strFileName )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Save the filename.
    wxFileName oFileName( strFileName );
    oFileName.Normalize();
    wxString strFileNameFullPath= oFileName.GetFullPath();

    // Open file.
    if( !m_oImage.LoadFile( strFileNameFullPath ) )
    {
        // Signal error;
        return false;
    }
    m_strImageFileName= strFileNameFullPath;
    SetTitle( m_strImageFileName + wxT( " - " ) + cstrAppName );

    // Get the image count and reset the page number.
    m_nImageCount= m_oImage.GetImageCount( m_strImageFileName );
    m_nPage= 0;

    // Update the page number info in the statusbar.
    SetStatusText( wxString::Format( wxT( "%d/%d" ),  m_nPage + 1, m_nImageCount ), 2 );

    // Get the image resolution (or use the last acquired one by not
    // changing the resolution).
    if( m_oImage.HasOption( wxIMAGE_OPTION_RESOLUTION ) )
    {
        m_nResolution= m_oImage.GetOptionInt( wxIMAGE_OPTION_RESOLUTION );
    }
    else if( m_oImage.HasOption( wxIMAGE_OPTION_RESOLUTIONX ) )
    {
        m_nResolution= m_oImage.GetOptionInt( wxIMAGE_OPTION_RESOLUTIONX );
    }
    else if( m_oImage.HasOption( wxIMAGE_OPTION_RESOLUTIONY ) )
    {
        m_nResolution= m_oImage.GetOptionInt( wxIMAGE_OPTION_RESOLUTIONY );
    }

    // Get the image orientation.
    m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );

    // Show the image on the canvas window.
    SetBitmap();

    // Enable (additional) menu options after the 1st scan.
    EnableControls();

    // Add filename to file history.
    m_oFileHistory.AddFileToHistory( m_strImageFileName );

    // Signal success.
    return true;
}

bool wxIScanFrame::OnDocumentPageChanged()
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Load the image by name and page number.
    if( !m_oImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, m_nPage ) )
    {
        // Signal error.
        return false;
    }

    // Update the page number info in the statusbar.
    SetStatusText( wxString::Format( wxT( "%d/%d" ),  m_nPage + 1, m_nImageCount ), 2 );

    // Show the image
    SetBitmap();

    // Get the image direction.
    m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );

    // Update the menu and toolbar controls.
    EnableControls();

    // Signal success.
    return true;
}

void wxIScanFrame::SetBitmap()
{
    // Show busy cursor (until the automatic variable is destroyed).
    wxBusyCursor oWait;

    // Resize image (if necessary).
    switch( m_nFit )
    {
        case wxCANVASFITWIDTH   : {
                                      int x, nDummy;

                                      m_poCanvas->GetClientSize( &x, &nDummy );

                                      m_nScaleFactorX= (double)x / (double)m_oImage.GetWidth();

                                      m_poCanvas->SetBitmap( m_oImage.Scale(
                                          (int)( (double)m_oImage.GetWidth()  * m_nScaleFactorX ),
                                          (int)( (double)m_oImage.GetHeight() * m_nScaleFactorX ),
                                          wxIMAGE_QUALITY_HIGH ) );
                                  }
                                  break;
        case wxCANVASFITWINDOW  : {
                                      int x, y;
                                      double nScaleFactorY;

                                      m_poCanvas->GetClientSize( &x, &y );

                                      m_nScaleFactorX= (double)x / (double)m_oImage.GetWidth();
                                      nScaleFactorY= (double)y / (double)m_oImage.GetHeight();

                                      if( m_nScaleFactorX > nScaleFactorY )
                                      {
                                          m_nScaleFactorX= nScaleFactorY;
                                      }
                                      m_poCanvas->SetBitmap( m_oImage.Scale(
                                          (int)( (double)m_oImage.GetWidth()  * m_nScaleFactorX ),
                                          (int)( (double)m_oImage.GetHeight() * m_nScaleFactorX ),
                                          wxIMAGE_QUALITY_HIGH ) );
                                  }
                                  break;
        case wxCANVASFULLSIZE   : // Nothing to do for this is the default.
        default                 : m_poCanvas->SetBitmap( m_oImage );
                                  break;
    }
}

wxImage wxIScanFrame::GetImageSelection()
{
    if( m_poCanvas->IsSelected() )
    {
        wxRect oSelection= m_poCanvas->GetSelection();

        oSelection.x     = int( (double)oSelection.x      / m_nScaleFactorX );
        oSelection.y     = int( (double)oSelection.y      / m_nScaleFactorX );
        oSelection.width = int( (double)oSelection.width  / m_nScaleFactorX );
        oSelection.height= int( (double)oSelection.height / m_nScaleFactorX );

        int nWidth= m_oImage.GetWidth();
        if( nWidth < ( oSelection.x + oSelection.width ) )
        {
            oSelection.width= nWidth - oSelection.x;
        }

        int nHeight= m_oImage.GetHeight();
        if( nHeight < ( oSelection.y + oSelection.height ) )
        {
            oSelection.height= nHeight - oSelection.y;
        }

        return m_oImage.GetSubImage( oSelection );
    }
    return m_oImage;
}

void wxIScanFrame::ScanImage()
{
    // Object representing the scanner.
    wxScan oScanner;

    // Check if the scanner object initialization was successful.
    if( !oScanner.IsOK() )
    {
        ::wxLogError( _( "Cannot initialize SANE object." ) );
        return;
    }

    // Scan an image, ...
    if( !oScanner.ScanImage( m_oImage ) )
    {
        ::wxLogError( _( "An error occured while scanning." ) );
        return;
    }

    // ... get its resolution, ...
    m_nResolution= oScanner.GetResolution();

    // ... apply an image profile (crop, convert to monochrome etc.), and ...
    if( m_bAutoApplyProfile )
    {
        ApplyProfile();
    }

    // ... get the image orientation, and ...
    m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );

    // ... show the image on the canvas window.
    SetBitmap();

    // Reset the image count, the page number and the image filename.
    ResetFileName();
}

void wxIScanFrame::SetProfile( int nProfileIndex )
{
    GetMenuBar()->Check( ID_DOCUMENTPROFILE0 + m_nProfileIndex, false );
    m_nProfileIndex= nProfileIndex;
    GetMenuBar()->Check( ID_DOCUMENTPROFILE0 + m_nProfileIndex, true  );
}

void wxIScanFrame::ApplyProfile()
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Load image conversion profile, and ...
    wxIScanImageProfile oImageProfile( m_nProfileIndex, m_nResolution );

    // ... apply the conversions.
    m_oImage= oImageProfile.ApplyToImage( m_oImage );
}

wxString wxIScanFrame::OCR( const wxString &strTempFileName )
{
    // Check if there are valid commands for ocr functions.
    if( !( m_bCmdConvertEnable && m_bCmdOcrEnable && m_bCmdCleanUpEnable ) )
    {
#if __DEBUG__
        ::wxLogMessage( wxT( "wxIScanFrame::OCR - At least one OCR relevant command is not yet valid set." ) );
#endif
        return wxEmptyString;
    }


    // ...
    wxString strCleanUpCommand= wxString::Format( m_strCmdCleanUp,
                                                  strTempFileName.c_str(),
                                                  strTempFileName.c_str() );
#if __DEBUG__
    ::wxLogMessage( wxT( "wxIScanFrame::OCR - strCleanUpCommand == '%s'" ), strCleanUpCommand.c_str() );
#endif

    wxString strCommand= wxString::Format( m_strCmdConvert,
                                           strTempFileName.c_str(),
                                           strTempFileName.c_str() );
#if __DEBUG__
    ::wxLogMessage( wxT( "wxIScanFrame::OCR - strCommand == '%s'" ), strCommand.c_str() );
#endif
    if( ::wxExecute( strCommand, wxEXEC_SYNC ) != 0 )
    {
        ::wxLogMessage( wxString( wxT( "wxIScanFrame::OCR - " ) )
                        + _( "Creating monochrome file failed." ) );
        ::wxExecute( strCleanUpCommand, wxEXEC_SYNC );
        return wxEmptyString;
    }

    // ...
    strCommand= wxString::Format( m_strCmdOcr,
                                  strTempFileName.c_str(),
                                  strTempFileName.c_str() );
#if __DEBUG__
    ::wxLogMessage( wxT( "wxIScanFrame::OCR - strCommand == '%s'" ), strCommand.c_str() );
#endif
    if( ::wxExecute( strCommand, wxEXEC_SYNC ) != 0 )
    {
        ::wxLogMessage( wxString( wxT( "wxIScanFrame::OCR - " ) )
                        + _( "OCR failed." ) );
        ::wxExecute( strCleanUpCommand, wxEXEC_SYNC );
        return wxEmptyString;
    }

    // Clean up.
    ::wxExecute( strCleanUpCommand, wxEXEC_SYNC );

    // Return filename of the created text file.
    return strTempFileName + wxT( ".txt" ); // TODO
}

#if __WXPDFDOC__
bool wxIScanFrame::CreateNewPdfDocument()
{
    // Delete any old PDF documents.
    if( m_poPdfDoc  )
    {
        delete m_poPdfDoc;
    }
    SetStatusText( wxString( wxT( "PDF: " ) ) + _( "empty" ), 1 );

    // Create a new PDF document in portrait mode on A4 pager using mm
    // as measurement unit.
    m_poPdfDoc= new wxPdfDocument( wxPORTRAIT,
                                   wxString( _T( "mm" ) ),
                                   wxPAPER_A4 );
    m_nPdfPages= 0;
    m_bPdfDirty= false;
    if( !m_poPdfDoc )
    {
        ::wxLogError( _( "Cannot create wxPdfDocument." ) );
    }
    m_poPdfDoc->SetCreator( ::wxGetApp().GetAppName() );
    m_poPdfDoc->SetAutoPageBreak( false );
    m_poPdfDoc->SetCompression( true );
    return true;
}

bool wxIScanFrame::AddPdfPage()
{
    if( !m_poPdfDoc )
    {
        ::wxLogError( _( "wxPdfDocument is NULL" ) );
        return false;
    }

    // Save image in a temporary JPEG file.
    wxString strTempFileName= wxFileName::CreateTempFileName( wxT( "" ) );

    m_oImage.SaveFile( strTempFileName, m_strPdfBitmapMimeType );

#if defined( __WXSCANDEBUG__ ) && __WXSCANDEBUG__ >= 1
    ::wxLogMessage( wxT( "PDF PARAMETERS" ) );
    ::wxLogMessage( wxT( "\tGetPageWidth == %.2f mm (%f)"  ), m_poPdfDoc->GetPageWidth(),  m_poPdfDoc->GetPageWidth()  / m_oImage.GetWidth() );
    ::wxLogMessage( wxT( "\tGetPageHeight == %.2f mm (%f)" ), m_poPdfDoc->GetPageHeight(), m_poPdfDoc->GetPageHeight() / m_oImage.GetHeight() );
    ::wxLogMessage( wxT( "\tGetImageScale == %f (new: %f)" ), m_poPdfDoc->GetImageScale(), (double)m_nResolution / 72.0 );
    ::wxLogMessage( wxT( "\tGetScaleFactor == %f" ), m_poPdfDoc->GetScaleFactor() );
    ::wxLogMessage( wxT( "\tm_nResolution == %.2f DPI (%.2f/mm)" ), (double)m_nResolution, (double)m_nResolution / 25.4 );
    ::wxLogMessage( wxT( "============" ) );
    ::wxSafeYield();
#endif

    // Add a page, ...
    m_poPdfDoc->AddPage( m_bPortrait ? wxPORTRAIT : wxLANDSCAPE );

    // ... do some OCR and include the recognized text if enabled, and ...
    if( m_bOCR )
    {
        wxString strOcrTextFileName= OCR( strTempFileName );

        if( !strOcrTextFileName.IsEmpty() )
        {
            m_poPdfDoc->SetFont( wxT( "Times" ), wxT( "" ) ,8 );
            m_poPdfDoc->SetTextColour( 0.0, 0.0, 0.0, 0.0 );
            m_poPdfDoc->MultiCell( 0, 3, GetTextFromFile( strOcrTextFileName ), wxPDF_BORDER_NONE, wxPDF_ALIGN_LEFT, 1 );

            // ...
            ::wxRemoveFile( strOcrTextFileName );
        }
    }

    // ... put the image onto the page.
    m_poPdfDoc->SetImageScale(  (double)m_nResolution / 72.0 );
    m_poPdfDoc->Image( strTempFileName, 0 , 0, 0, 0, m_strPdfBitmapMimeType );

    // Remove temporary file.
    ::wxRemoveFile( strTempFileName );

    // Incremement the page counter and mark the PDF document as 'dirty'.;
    m_nPdfPages++;
    m_bPdfDirty= true;
    SetStatusText( wxString( wxT( "PDF: " ) ) + wxString::Format( _("Page %d" ), m_nPdfPages), 1 );

    // Signal success.
    return true;
}
#endif

wxString wxIScanFrame::GetTextFromFile( const wxString &strFilename )
{
    // Open a textfile, ...
    wxFile oFile( strFilename );

    if( !oFile.IsOpened() )
    {
        // Return an empty string on failure.
        return wxEmptyString;
    }

    // create a buffer of file size, ...
    wxFileOffset nLength= oFile.Length();
    char *pcBuffer= new char[nLength];

    if( !pcBuffer )
    {
        // Return an empty string on failure.
        return wxEmptyString;
    }

    // read the file's content, ...
    oFile.Read( pcBuffer, nLength );

    wxString strText= wxString::FromUTF8( pcBuffer, nLength );

    delete[] pcBuffer;

    // and return the text.
    return strText;
}

void wxIScanFrame::CopyText2Clipboard( const wxString &strText )
{
    // Create a text data object for the clipboard.
    wxTextDataObject *poTextData=new wxTextDataObject;

    if( !poTextData )
    {
        ::wxLogError( wxString( wxT( "wxIScanFrame::CopyText2Clipboard -- " ) )
                      + _( "Failed to create clipboard text data object." ) );
        return;
    }

	// Copy the text to the text data object for the clipboard.
    poTextData->SetText( strText );

	// Open the clipboard and copy the text to the clipboard.
    if( !wxTheClipboard->Open()
        || !wxTheClipboard->SetData( poTextData ) )
    {
        ::wxLogError( wxString( wxT( "wxIScanFrame::CopyText2Clipboard -- " ) )
                      + _( "Failed to copy text to clipboard." ) );
    }

	// Close the clipboard (and release it).
    wxTheClipboard->Close();
}

void wxIScanFrame::CopyImage2Clipboard( const wxImage &oImage )
{
    // Create a image data object for the clipboard.
    wxBitmapDataObject *poBitmapData=new wxBitmapDataObject;

    if( !poBitmapData )
    {
        ::wxLogError( wxString( wxT( "wxIScanFrame::CopyImage2Clipboard -- " ) )
                      + _( "Failed to create clipboard image data object." ) );
        return;
    }

	// Copy the image to the image data object for the clipboard.
    poBitmapData->SetBitmap( oImage );

	// Open the clipboard and copy the image to the clipboard.
    if( !wxTheClipboard->Open()
        || !wxTheClipboard->SetData( poBitmapData ) )
    {
        ::wxLogError( wxString( wxT( "wxIScanFrame::CopyImage2Clipboard -- " ) )
                      + _( "Failed to copy image to clipboard." ) );
    }

	// Close the clipboard (and release it).
    wxTheClipboard->Close();
}

void wxIScanFrame::ResetFileName()
{
    // Reset the file name.
    m_strImageFileName= _( "unknown.jpg" );
    SetTitle( cstrAppName );

    // Reset the image count, the page number and the image filename.
    m_nImageCount= 1;
    m_nPage= 0;

    // Update the page number info in the statusbar.
    SetStatusText( wxString::Format( wxT( "%d/%d" ),  m_nPage + 1, m_nImageCount ), 2 );
}

// Restore program settings from config file.
//
void wxIScanFrame::RestoreSettings()
{
    // Reset window settings.
    wxConfigBase *poConfig = wxConfigBase::Get();
    poConfig->SetRecordDefaults();
    poConfig->SetPath( wxT( "/" )  );
    poConfig->SetPath( cstrwxFrame );

    int x= poConfig->Read( wxT( "x" ),   5 );
    int y= poConfig->Read( wxT( "y" ),   5 );
    int w= poConfig->Read( wxT( "w" ), 630 );
    int h= poConfig->Read( wxT( "h" ), 470 );
    bool bMaximized= (bool)poConfig->Read( wxT( "Maximized"  ), (long)false );
    bool bIconized= (bool)poConfig->Read( wxT( "Iconized"  ), (long)false );

    // Reset window position, size and maximization state.
    Move( x, y );
    SetSize( w, h );
    Maximize( bMaximized );
    Iconize( bIconized );

    // Reset sash position and splitting option.
    int nHSashPos= poConfig->Read( wxT( "HSashPos" ), h / 5 );

    m_poHSplitterWindow->SetSashPosition( nHSashPos );
//    m_poHSplitterWindow->SetSashGravity( 1.0 );

    // Reset ...
    m_nFit= poConfig->Read( wxT( "Fit" ), wxCANVASFITWINDOW );

    // Reset bitmap mimetype for pdf export.
    m_strPdfBitmapMimeType= poConfig->Read( wxT( "PdfBitmapMimeType" ), wxT( "image/jpeg" ) );

    // Get OCR and tts configuration.
    m_bAutoApplyProfile= (bool)poConfig->Read( wxT( "AutoApplyProfile" ), (long)false );
    m_bOCR= (bool)poConfig->Read( wxT( "OCR" ), (long)false );
    m_bCmdConvertEnable= poConfig->Read( wxT( "CmdConvertEnable" ), (long)false );
    m_strCmdConvert= poConfig->Read( wxT( "CmdConvert" ), wxT( "convert %s -compress Group4 %s.tif" ) );
    m_bCmdOcrEnable= poConfig->Read( wxT( "CmdOcrEnable" ), (long)false );
    m_strCmdOcr= poConfig->Read( wxT( "CmdOcr" ), wxT( "tesseract %s.tif %s" ) );
    m_bCmdCleanUpEnable= poConfig->Read( wxT( "CmdCleanUpEnable" ), (long)false );
    m_strCmdCleanUp= poConfig->Read( wxT( "CmdCleanUp" ), wxT( "rm %s.tif" ) );
    m_bCmdTtsEnable= poConfig->Read( wxT( "CmdTtsEnable" ), (long)false );
    m_strCmdTts= poConfig->Read( wxT( "CmdTts" ), wxT( "espeak -f %s" ) );
    m_bCmdEditEnable= poConfig->Read( wxT( "CmdEditEnable" ), (long)false );
    m_strCmdEdit= poConfig->Read( wxT( "CmdEdit" ), wxT( "scite %s" ) );

    // Restore the standard resolution.
    m_nResolution= poConfig->Read( wxT( "Resolution" ), 300 );
}

// Save program settings in config file.
//
void wxIScanFrame::SaveSettings()
{
    // Save configuration data.
    wxConfigBase *poConfig= wxConfigBase::Get();

    poConfig->SetPath( wxT( "/" ) );
    poConfig->SetPath( cstrwxFrame );

	// Save iconifization state.
    poConfig->Write( wxT( "Iconized" ),  (long)IsIconized() );
    if( IsIconized() )
    {
        Iconize( false );
    }

	// Save maximization state.
    poConfig->Write( wxT( "Maximized" ),  (long)IsMaximized() );
    if( IsMaximized() )
    {
        Maximize( false );
    }

    // Save window position and size.
	//
    int x, y, w, h;

    GetSize( &w, &h );
    GetPosition( &x, &y );
    poConfig->Write( wxT( "x" ), (long)x );
    poConfig->Write( wxT( "y" ), (long)y );
    poConfig->Write( wxT( "w" ), (long)w );
    poConfig->Write( wxT( "h" ), (long)h );

    // Save the sash position.
    poConfig->Write( wxT( "HSashPos" ), (long)m_poHSplitterWindow->GetSashPosition() );

    // Save the resolution as standard resolution.
    poConfig->Write( wxT( "Resolution" ), (long)m_nResolution );

    // ...
    poConfig->Write( wxT( "Fit" ), (long)m_nFit );

    // ...
    poConfig->Write( wxT( "PdfBitmapMimeType" ), m_strPdfBitmapMimeType );

    // ...
    poConfig->Write( wxT( "AutoApplyProfile" ), (long)m_bAutoApplyProfile );
    poConfig->Write( wxT( "OCR" ), (long)m_bOCR );
    poConfig->Write( wxT( "CmdConvertEnable" ), (long)m_bCmdConvertEnable );
    poConfig->Write( wxT( "CmdConvert" ), m_strCmdConvert );
    poConfig->Write( wxT( "CmdOcrEnable" ), (long)m_bCmdOcrEnable );
    poConfig->Write( wxT( "CmdOcr" ), m_strCmdOcr );
    poConfig->Write( wxT( "CmdCleanUpEnable" ), (long)m_bCmdCleanUpEnable );
    poConfig->Write( wxT( "CmdCleanUp" ), m_strCmdCleanUp );
    poConfig->Write( wxT( "CmdTtsEnable" ), (long)m_bCmdTtsEnable );
    poConfig->Write( wxT( "CmdTts" ), m_strCmdTts );
    poConfig->Write( wxT( "CmdEditEnable" ), (long)m_bCmdEditEnable );
    poConfig->Write( wxT( "CmdEdit" ), m_strCmdEdit );

    // ...
    poConfig->Write( wxT( "ProfileIndex" ), m_nProfileIndex );

    // Save file history to config file.
    poConfig->SetPath( wxT( "/Files" ) );
    m_oFileHistory.Save( *poConfig );
}

// Enable or disable controls according to the progam state.
//
void wxIScanFrame::EnableControls()
{
    // Enable/disable menu items (on first call).
    bool bImageOk= !m_bUILocked && m_oImage.IsOk();

    GetMenuBar()->Enable(     ID_FILESAVEAS,                bImageOk );
    GetToolBar()->EnableTool( ID_FILESAVEAS_TB,             bImageOk );
    GetMenuBar()->Enable(     ID_FILEPRINT,                 bImageOk );
    GetToolBar()->EnableTool( ID_FILEPRINT_TB,              bImageOk );
    GetMenuBar()->Enable(     ID_FILEPRINTPREVIEW,          bImageOk );
    GetMenuBar()->Enable(     ID_FILEPRINTSETUP,            bImageOk );
    GetMenuBar()->Enable(     ID_EDITCOPY,                  bImageOk );
#if __WXPDFDOC__
    GetMenuBar()->Enable(     ID_PDFADDPAGE,                bImageOk );
    GetToolBar()->EnableTool( ID_PDFADDPAGE_TB,             bImageOk );
#else
    GetMenuBar()->Enable(     ID_PDFADDPAGE,                false );
    GetToolBar()->EnableTool( ID_PDFADDPAGE_TB,             false );
#endif // __WXPDFDOC__
    GetMenuBar()->Enable(     ID_DOCUMENTCROP,              bImageOk );
    GetMenuBar()->Enable(     ID_DOCUMENTROTATE90LEFT,      bImageOk );
    GetMenuBar()->Enable(     ID_DOCUMENTROTATE90RIGHT,     bImageOk );
    GetMenuBar()->Enable(     ID_DOCUMENTAPPLYPROFILE,      bImageOk );

#if __WXPDFDOC__
    // Enable/disable PDF file save controls.
    bool bCanSavePdfFile= !m_bUILocked && ( m_nPdfPages > 0 );

    GetMenuBar()->Enable(     ID_PDFSAVEASPDF,              bCanSavePdfFile );
    GetToolBar()->EnableTool( ID_PDFSAVEASPDF_TB,           bCanSavePdfFile );
#else
    GetMenuBar()->Enable(     ID_PDFSAVEASPDF,              false );
    GetToolBar()->EnableTool( ID_PDFSAVEASPDF_TB,           false );
#endif // __WXPDFDOC__

    // ...
    bool bCopyText= bImageOk
                && m_bCmdConvertEnable
                && m_bCmdOcrEnable
                && m_bCmdCleanUpEnable;

    GetMenuBar()->Enable(     ID_DOCUMENTCOPYTEXT,         bCopyText );

    // ...
    GetMenuBar()->Enable(     ID_EDITSETTINGS,             m_bCmdEditEnable );

    // Enabel text-to-speach controls
    bool bTTS= bImageOk
                && m_bCmdConvertEnable
                && m_bCmdOcrEnable
                && m_bCmdTtsEnable
                && m_bCmdCleanUpEnable
                && ( m_poRunningProcess ? false : true );

    GetMenuBar()->Enable(     ID_DOCUMENTREADTEXT,          bTTS );
    GetToolBar()->EnableTool( ID_DOCUMENTREADTEXT_TB,       bTTS );

    // Enable/disable page controls
    bool bNotFirstPage= ( m_nPage > 0 );
    bool bNotLastPage=  ( m_nPage < ( m_nImageCount - 1 ) );

    GetMenuBar()->Enable(     ID_DOCUMENTFIRSTPAGE,         bImageOk && bNotFirstPage );
    GetToolBar()->EnableTool( ID_DOCUMENTFIRSTPAGE_TB,      bImageOk && bNotFirstPage );
    GetMenuBar()->Enable(     ID_DOCUMENTPREVIOUSPAGE,      bImageOk && bNotFirstPage );
    GetToolBar()->EnableTool( ID_DOCUMENTPREVIOUSPAGE_TB,   bImageOk && bNotFirstPage );
    GetMenuBar()->Enable(     ID_DOCUMENTNEXTPAGE,          bImageOk && bNotLastPage  );
    GetToolBar()->EnableTool( ID_DOCUMENTNEXTPAGE_TB,       bImageOk && bNotLastPage  );
    GetMenuBar()->Enable(     ID_DOCUMENTLASTPAGE,          bImageOk && bNotLastPage  );
    GetToolBar()->EnableTool( ID_DOCUMENTLASTPAGE_TB,       bImageOk && bNotLastPage  );

    // Enable process killer.
    GetMenuBar()->Enable(     ID_CMDSTOP,                   m_poRunningProcess );
    GetToolBar()->EnableTool( ID_CMDSTOP_TB,                m_poRunningProcess );

    // ...
    GetMenuBar()->Enable(     ID_FILEOPEN,                  !m_bUILocked );
    GetToolBar()->EnableTool( ID_FILEOPEN_TB,               !m_bUILocked );
    GetMenuBar()->Enable(     ID_FILESCANIMAGE,             !m_bUILocked );
    GetToolBar()->EnableTool( ID_FILESCANIMAGE_TB,          !m_bUILocked );
    GetMenuBar()->Enable(     ID_EDITPASTE,                 !m_bUILocked );

    // ... file history... .
    for( unsigned i= 0; i <  m_oFileHistory.GetCount(); i++ )
    {
        GetMenuBar()->Enable( m_oFileHistory.GetBaseId() + i, !m_bUILocked );
    }

    // Disable (by compiler option) not implemented controls .
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    GetMenuBar()->Enable(     ID_PDFBEGINPDF,               !m_bUILocked );
    GetToolBar()->EnableTool( ID_PDFBEGINPDF_TB,            !m_bUILocked );
#else
    GetToolBar()->EnableTool( ID_PDFBEGINPDF_TB,            false );
    GetToolBar()->EnableTool( ID_PDFBEGINPDF_TB,            false );
#endif
}

// ...
//
// NOTE: This function paints the image while printing or print previewing!
// NOTE: The painting on the screen is done by void wxIViewCanvas::Paint( wxDC& oDc )!
//
bool wxIScanFrame::Paint( wxDC& oDc )
{
    // If there is a bitmap connected to this canvas window (re)paint it.
    if( oDc.Ok() && m_oPrintImage.Ok() )
    {
        // Repaint the bitmap.
        oDc.DrawBitmap( wxBitmap( m_oPrintImage ), 0, 0 );
        return true;
    }
    else
    {
        return false;
    }
}

// ...
//
// NOTE: This function returns the image extents in mm while printing or
//       print previewing!
//
bool wxIScanFrame::GetPageSizeMM( int *pnWidth, int *pnHeight )
{
     if( m_oPrintImage.Ok() )
     {
         (*pnWidth)=  (int)( (double)m_oPrintImage.GetWidth()  / (double)m_nResolution * 25.4 );
         (*pnHeight)= (int)( (double)m_oPrintImage.GetHeight() / (double)m_nResolution * 25.4 );
         return true;
     }
     else
     {
         return false;
     }
}

// ...
//
bool wxIScanFrame::GotoPage( int nPage )
{
    // NOTE: The wxPrintOut object passes pages one-based. We need it
    //        zero-based. Therefore we decrement the page number by 1.
    //
    nPage--;

    // Avoid loading an already loaded image.
    if( nPage == m_nPage )
    {
        m_oPrintImage= m_oImage;
        return true;
    }
    else
    {
        return m_oPrintImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, nPage );
    }
}

