/***************************************************************
 * Name:      wxiscanframe.h
 * Purpose:   Defines Application Frame
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANMAIN_H
#define WXISCANMAIN_H

//(*Headers(wxIScanFrame)
#include <wx/toolbar.h>
#include <wx/menu.h>
#include <wx/scrolwin.h>
#include <wx/splitter.h>
#include <wx/statusbr.h>
#include <wx/frame.h>
#include <wx/textctrl.h>
//*)

// Additional wxWidgets header files
#include <wx/docview.h>
#include <wx/process.h>

// Private header files.
#include "wxextfiledroptarget.h"
#include "wxiviewcanvas.h"
#include "wxiviewpaint.h"

// IDs
#define wxISCAN_PROCESS     1


//////////////////////////////////////////////////////////
// Predefined classes
//
#if __WXPDFDOC__
    class wxPdfDocument;
#endif // __WXPDFDOC__
class wxExtProcess;


//////////////////////////////////////////////////////////
// Class wxIScanFrame
//
class wxIScanFrame: public wxFrame, public wxIViewPaintBase
{
    public:
        enum
        {
            wxCANVASFULLSIZE,
            wxCANVASFITWIDTH,
            wxCANVASFITWINDOW
        };

    public:
        wxIScanFrame(wxWindow* parent,wxWindowID id = -1);
        virtual ~wxIScanFrame();

    public:
        virtual bool CanClose( bool bCanVeto= true );               ///< ...
        virtual bool FileOpen( const wxString &strFileName );       ///< ...
        virtual bool OnDocumentPageChanged();                       ///< ...
        virtual void SetBitmap();                                   ///< ...
        virtual wxImage GetImageSelection();                        ///< ...
        virtual void ScanImage();                                   ///< ...
        virtual void SetProfile( int nProfileIndex );               ///< ...
        virtual void ApplyProfile();                                ///< ...
        virtual wxString OCR( const wxString &strTempFileName );    ///< ...
#if __WXPDFDOC__
        virtual bool CreateNewPdfDocument();                        ///< ...
        virtual bool AddPdfPage();                                  ///< ...
        virtual bool IsPdfDirty(){ return m_bPdfDirty; }            ///< ...
#endif // __WXPDFDOC__
        virtual wxString GetTextFromFile( const wxString &strFilename ); ///< ...
        virtual void CopyText2Clipboard( const wxString &strText ); ///< Copies the given text to the sytem clipboard.
        virtual void CopyImage2Clipboard( const wxImage &oImage );  ///< Copies the given image to the sytem clipboard.
        virtual void ResetFileName();                               ///< ...
        virtual void RestoreSettings();                             ///< ...
        virtual void SaveSettings();                                ///< ...
        virtual void EnableControls();                              ///< ...
        virtual void LockUI( bool bLock= true ){ m_bUILocked= bLock; EnableControls(); } ///< ...
        virtual void UnlockUI(){ LockUI( false ); }                 ///< ...

    public:
        // Printer handling (introduced by base class wxIViewPaintBase)
        virtual void AdjustOrientation(){ m_oPrintData.SetOrientation( m_bPortrait ? wxPORTRAIT : wxLANDSCAPE ); }
        virtual bool GetPageSizeMM( int *pnWidth, int *pnHeight );
        virtual int  GetMaxPage(){ return m_nImageCount; }
        virtual bool GotoPage( int nPage );
        virtual bool Paint( wxDC& oDc );

    private:
        // Member variables
        bool m_bUILocked;                                           ///< ...
        wxImage m_oImage;                                           ///< Image object.
        wxImage m_oPrintImage;                                      ///< Image object for printing.
        wxString m_strImageFileName;                                ///< ...
        double m_nScaleFactorX;                                     ///< ...
        int m_nPage;                                                ///< ...
        int m_nImageCount;                                          ///< ...
        int m_nResolution;                                          ///< ...
        int m_bPortrait;                                            ///< ...

        // Options
        int m_nFit;                                                 ///< ...
        bool m_bAutoApplyProfile;                                   ///< ...
        int m_nProfileIndex;                                        ///< ...
        wxString m_strPdfBitmapMimeType;                            ///< Image mimetype for pdf export.
        bool m_bOCR;                                                ///< ...
        bool m_bCmdConvertEnable;                                   ///< ...
        wxString m_strCmdConvert;                                   ///< ...
        bool m_bCmdOcrEnable;                                       ///< ...
        wxString m_strCmdOcr;                                       ///< ...
        bool m_bCmdCleanUpEnable;                                   ///< ...
        wxString m_strCmdCleanUp;                                   ///< ...
        bool m_bCmdTtsEnable;                                       ///< ...
        wxString m_strCmdTts;                                       ///< ...
        bool m_bCmdEditEnable;                                      ///< ...
        wxString m_strCmdEdit;                                      ///< ...

        // File history
        wxFileHistory m_oFileHistory;                               ///< History of last opened files

        // Printing support.
        wxPrintData m_oPrintData;                                   ///< ...

#if __WXPDFDOC__
        // PDF support
        wxPdfDocument *m_poPdfDoc;                                  ///< ...
        int m_nPdfPages;                                            ///< ...
        bool m_bPdfDirty;                                           ///< ...
#endif // __WXPDFDOC__

        // ...
        wxExtProcess *m_poRunningProcess;                           ///< A concurrenct running process.

    private:
        //(*Handlers(wxIScanFrame)
        void OnFileQuit(wxCommandEvent& event);
        void OnFileScanImage(wxCommandEvent& event);
        void OnFileOpen(wxCommandEvent& event);
        void OnFileSaveAs(wxCommandEvent& event);
        void OnFilePrint( wxCommandEvent &oEvent );             ///< Handle menu command "File|Print".
        void OnFilePrintPreview( wxCommandEvent &oEvent );      ///< Handle menu command "File|Print Preview".
        void OnFilePrintSetup( wxCommandEvent &oEvent );        ///< Handle menu command "File|Page Setup ".
        void OnEditCopy(wxCommandEvent& event);
        void OnPdfBeginPdf(wxCommandEvent& event);
        void OnDocumentCopyText(wxCommandEvent& event);
        void OnDocumentReadText(wxCommandEvent& event);
        void OnDocumentFirstPage(wxCommandEvent& event);
        void OnDocumentPreviousPage(wxCommandEvent& event);
        void OnDocumentNextPage(wxCommandEvent& event);
        void OnDocumentLastPage(wxCommandEvent& event);
        void OnDocumentRotate90Left(wxCommandEvent& event);
        void OnDocumentRotate90Right(wxCommandEvent& event);
        void OnDocumentApplyProfile(wxCommandEvent& event);
        void OnPdfAddPage(wxCommandEvent& event);
        void OnPdfSavePdf(wxCommandEvent& event);
        void OnHelpAbout(wxCommandEvent& event);
        void OnCmdStop(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnEditPaste(wxCommandEvent& event);
        void OnEditSettings(wxCommandEvent& event);
        void OnEditSettingsFile(wxCommandEvent& event);
        void OnDocumentCrop(wxCommandEvent& event);
        //*)
        void OnDocumentProfilesChooseProfile(wxCommandEvent& oEvent);
        void OnMRUFiles( wxCommandEvent& oEvent );                  ///< Handle menu command "File|[MOST RECENTLY USES FILES]".
        void OnFileDrop( wxExtFileDropEvent& oEvent );              ///< ...
        void OnCanvasSize( wxIViewCanvasSizeEvent& oEvent );        ///< ...
        void OnProcessTerminated( wxProcessEvent& oEvent );         ///< ...

        //(*Identifiers(wxIScanFrame)
        static const long ID_CANVAS;
        static const long ID_LOGTEXTCTRL;
        static const long ID_HSPLITTERWINDOW;
        static const long ID_FILESCANIMAGE;
        static const long ID_FILEOPEN;
        static const long ID_FILESAVEAS;
        static const long ID_FILEPRINT;
        static const long ID_FILEPRINTPREVIEW;
        static const long ID_FILEPRINTSETUP;
        static const long ID_FILEQUIT;
        static const long ID_EDITCOPY;
        static const long ID_EDITPASTE;
        static const long ID_EDITSETTINGS;
        static const long ID_EDITSETTINGSFILE;
        static const long ID_DOCUMENTCOPYTEXT;
        static const long ID_DOCUMENTREADTEXT;
        static const long ID_CMDSTOP;
        static const long ID_DOCUMENTFIRSTPAGE;
        static const long ID_DOCUMENTPREVIOUSPAGE;
        static const long ID_DOCUMENTNEXTPAGE;
        static const long ID_DOCUMENTLASTPAGE;
        static const long ID_DOCUMENTCROP;
        static const long ID_DOCUMENTROTATE90LEFT;
        static const long ID_DOCUMENTROTATE90RIGHT;
        static const long ID_DOCUMENTAPPLYPROFILE;
        static const long ID_MENUDOCUMENTPROFILES;
        static const long ID_PDFBEGINPDF;
        static const long ID_PDFADDPAGE;
        static const long ID_PDFSAVEASPDF;
        static const long ID_HELPABOUT;
        static const long ID_STATUSBAR;
        static const long ID_FILESCANIMAGE_TB;
        static const long ID_FILEOPEN_TB;
        static const long ID_FILESAVEAS_TB;
        static const long ID_FILEPRINT_TB;
        static const long ID_PDFBEGINPDF_TB;
        static const long ID_PDFADDPAGE_TB;
        static const long ID_PDFSAVEASPDF_TB;
        static const long ID_DOCUMENTFIRSTPAGE_TB;
        static const long ID_DOCUMENTPREVIOUSPAGE_TB;
        static const long ID_DOCUMENTNEXTPAGE_TB;
        static const long ID_DOCUMENTLASTPAGE_TB;
        static const long ID_DOCUMENTREADTEXT_TB;
        static const long ID_CMDSTOP_TB;
        static const long ID_TOOLBAR;
        //*)
        static const long ID_DOCUMENTPROFILE0;


        //(*Declarations(wxIScanFrame)
        wxIViewCanvas* m_poCanvas;
        wxMenuItem* MenuItem1;
        wxToolBarToolBase* ToolBarItem5;
        wxMenu* Menu1;
        wxToolBarToolBase* ToolBarItem11;
        wxToolBarToolBase* ToolBarItem6;
        wxToolBarToolBase* ToolBarItem12;
        wxStatusBar* m_poStatusBar;
        wxToolBarToolBase* ToolBarItem7;
        wxToolBarToolBase* ToolBarItem2;
        wxToolBarToolBase* ToolBarItem9;
        wxToolBarToolBase* ToolBarItem10;
        wxToolBarToolBase* ToolBarItem4;
        wxToolBarToolBase* ToolBarItem1;
        wxToolBarToolBase* ToolBarItem3;
        wxSplitterWindow* m_poHSplitterWindow;
        wxTextCtrl* m_poLogTextCtrl;
        wxToolBarToolBase* ToolBarItem8;
        //*)

        DECLARE_EVENT_TABLE()
};

#endif // WXISCANMAIN_H
