/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxiviewcanvas.h
// Author:      Daniel Nell
// Created:     2005/10/18
// Copyright:   (c) Daniel Nell
// License:     wxWindows license
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXIVIEWCANVAS_H
#define __WXIVIEWCANVAS_H

/////////////////////////////////////////////////////////////////////////////
//  Constants
//
#define WXIVIEWCANVASDELAY  50


/////////////////////////////////////////////////////////////////////////////
//  Class wxIViewCanvasSizeEvent
//
BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_LOCAL_EVENT_TYPE( wxEVT_IVIEWCANVASSIZE, 0 )
END_DECLARE_EVENT_TYPES()

/////////////////////////////////////////////////////////////////////////////
//  Class wxIViewCanvasSizeEvent
//
/// \brief ... event.
///
/// ...
class wxIViewCanvasSizeEvent : public wxEvent
{
  public:
    /// Constructor
    wxIViewCanvasSizeEvent( wxEventType oType= wxEVT_IVIEWCANVASSIZE ) : wxEvent( 0, oType )
    {
        // Propagate event to the next event handling window.
        m_propagationLevel= wxEVENT_PROPAGATE_MAX;
    }

    /// Return a copy ot this event.
    wxEvent *Clone() const
    {
        return new wxIViewCanvasSizeEvent( *this );
    }
};

typedef void (wxEvtHandler::*wxIViewCanvasSizeEventEventFunction)( wxIViewCanvasSizeEvent& );

#define wxIViewCanvasSizeEventEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent( wxIViewCanvasSizeEventEventFunction, &func )

#define EVT_IVIEWCANVASSIZE(func) \
   wx__DECLARE_EVT0( wxEVT_IVIEWCANVASSIZE, wxIViewCanvasSizeEventEventHandler( func ) )


/////////////////////////////////////////////////////////////////////////////
//  class wxIViewCanvas
//
/// \brief The canvas window
///
/// ...
class wxIViewCanvas: public wxScrolledWindow
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poParent      a pointer to the dialog windows's parent window.
    /// \param nId           the window id, default: 1
    /// \param oPos          the window position, default: wxDefaultPosition
    /// \param oSize         the window size, default: wxDefaultSize
    /// \param nStyle        window style (default: wxHSCROLL | wxVSCROLL | wxSUNKEN_BORDER)
    /// \param strName       class name (default: wxT( "wxIViewCanvas" ))
    wxIViewCanvas( wxWindow *poParent,
                   wxWindowID nId= wxID_ANY,
                   const wxPoint &oPos= wxDefaultPosition,
                   const wxSize &oSize= wxDefaultSize,
                   long nStyle= wxHSCROLL | wxVSCROLL | wxSUNKEN_BORDER,
                   const wxString& strName= wxT( "wxIViewCanvas" )
                 );

    /// Destructor
    virtual ~wxIViewCanvas();

    /// \brief Handler for window (re)painting.
    ///
    ///  Creates and prepares a device context and calls Paint( wxDC& oDc ).
    void OnPaint( wxPaintEvent &oEvent );

    /// Repaints the image (using an internal bitmap).
    virtual void Paint( wxDC& oDc );

    /// Returns the internal bitmap.
    virtual wxBitmap& GetBitmap(){ return m_oBitmap; }

    /// Sets the internal bitmap.
    virtual void SetBitmap( const wxBitmap& oBitmap );

     /// Handle resizing.
    void OnSize( wxSizeEvent& oEvent );

    /// Handle timer events.
    void OnTimer( wxTimerEvent& oEvent );

    /// Handle mouse events.
    void OnMouse( wxMouseEvent& oEvent );

    /// Set options for the wxIViewCanvas.
    void SetOptions( int nOptions){ m_nOptions= nOptions; }

    /// Clears the current selection.
    void ClearSelection()
    {
//        RefreshRect( m_oSelection );
        Refresh();
        m_oSelection= wxRect( -1, -1, 0, 0 );
    }

    /// Gets the selection rectangle.
    wxRect& GetSelection(){ return m_oSelection; };

    /// Checks if a selection area exists.
    bool IsSelected()
    {
        return    ( m_oSelection.x >= 0 )
               && ( m_oSelection.y >= 0 )
               && ( m_oSelection.width > 1 )
               && ( m_oSelection.height > 1 ) ;
    }

  public:
    /// wxIViewCanvas options (use SetOption function to set).
    enum
    {
        wxICANVAS_MOUSE_SCROLL=  1,
        wxICANVAS_MOUSE_SELECTION= 2
    };

  protected:
    // Member variables
    int m_nOptions;             ///< Holds extended style info (options).
    wxBitmap m_oBitmap;         ///< Internal bitmap object for repainting.
    wxTimer m_oTimer;           ///< Timer for delay before sending resizing event to parent window.
    wxPoint m_oMousePosBegin;   ///< Save the starting point of a mouse movement.
    wxPoint m_oMousePosOld;     ///< Save former mouse position.
    wxRect  m_oSelection;       ///< Holds the selection.

  private:
    // Any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif // __WXIVIEWCANVAS_H
