/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxoptionsdialog.h
// Author:      Daniel Nell
// Created:     2006/02/16
// Copyright:   (c) Daniel Nell
// Licence:     wxWindows licence
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXOPTIONSDIALOG_H
#define __WXOPTIONSDIALOG_H

/////////////////////////////////////////////////////////////////////////////
//  Constants
//
#define WXOPTIONSDIALOGMAXOPTION   100
#define WXOPTIONSDIALOGIDOPTION    31101

/////////////////////////////////////////////////////////////////////////////
//  Forward declarations
//
class wxBoxSizer;
class wxCheckBox;

/////////////////////////////////////////////////////////////////////////////
//  class wxOptionBase
//
/// \brief The (abstract) option for an option dialog.
///
/// This class represents an (abstract) option used in an option
/// dialog.
class wxOptionBase
{
  public:
    /// Default constructor.
    wxOptionBase(){}

    /// Destructor.
    virtual ~wxOptionBase(){};

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer ) = 0; ///< ...
    virtual void StoreValue() = 0;                      ///< ...
};

/////////////////////////////////////////////////////////////////////////////
//  class wxOptionCheckBox
//
/// \brief The boolean option for an option dialog.
///
/// This class represents a boolean (checkbox) option used in an
/// option dialog.
class wxOptionCheckBox : public wxOptionBase
{
  public:
    /// Constructor.
    wxOptionCheckBox( wxWindow *poWindow, int nId, const wxString &strText, bool *pbValue );

    /// Destructor.
    virtual ~wxOptionCheckBox(){};

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer );
    virtual void StoreValue();

  protected:
      bool *m_pbValue;
      wxCheckBox *m_poCheckBox;
};

/////////////////////////////////////////////////////////////////////////////
//  class wxOptionTextBox
//
/// \brief The text input box for an option dialog.
///
/// This class represents a text (textbox) option used in an option
/// dialog.
class wxOptionTextBox : public wxOptionBase
{
  public:
    /// Constructor.
    wxOptionTextBox( wxWindow *poWindow, int nId, const wxString &strText, wxString *pstrValue, int nWidth= 300 );

    /// Destructor.
    virtual ~wxOptionTextBox(){};

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer );
    virtual void StoreValue();

  protected:
    wxWindow *m_poWindow;
    wxString m_strText;
    wxString *m_pstrValue;
    wxTextCtrl *m_poTextCtrl;
};

/////////////////////////////////////////////////////////////////////////////
//  class wxOptionsDialog
//
/// \brief The option dialog.
///
/// This class represents the application's options dialog.
class wxOptionsDialog : public wxDialog
{
  public:
    /// Default constructor.
    wxOptionsDialog(){}

    /// \brief Standard constructor
    ///
    /// \param poParentWindow       a pointer to parent window (the window owner).
    /// \param bCbI18n              ...
    /// \param bCbShowTip           ...
    /// \param bCbShowSplashscreen  ...
    wxOptionsDialog( wxWindow *poParentWindow );

    /// Destructor.
    virtual ~wxOptionsDialog();

  public:
    // Member functions
    virtual void SetupDialog();                                 ///< ...
    virtual bool AppendOption( wxOptionBase *poOption );  ///< ...

  protected:
    /// Handler for the OK button.
    void OnOk( wxCommandEvent& oEvent );

  protected:
    int m_nOptionCount;
    wxOptionBase *m_apoOptions[WXOPTIONSDIALOGMAXOPTION];

  private:
    // Any class wishing to process wxWindows events must use this macro.
    DECLARE_EVENT_TABLE()
};

#endif // __WXOPTIONSDIALOG_H

